/***************************************************************************//**
* \file SCBUART.c
* \version 3.20
*
* \brief
*  This file provides the source code to the API for the SCB Component.
*
* Note:
*
*******************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SCBUART_PVT.h"

#if (SCBUART_SCB_MODE_I2C_INC)
    #include "SCBUART_I2C_PVT.h"
#endif /* (SCBUART_SCB_MODE_I2C_INC) */

#if (SCBUART_SCB_MODE_EZI2C_INC)
    #include "SCBUART_EZI2C_PVT.h"
#endif /* (SCBUART_SCB_MODE_EZI2C_INC) */

#if (SCBUART_SCB_MODE_SPI_INC || SCBUART_SCB_MODE_UART_INC)
    #include "SCBUART_SPI_UART_PVT.h"
#endif /* (SCBUART_SCB_MODE_SPI_INC || SCBUART_SCB_MODE_UART_INC) */


/***************************************
*    Run Time Configuration Vars
***************************************/

/* Stores internal component configuration for Unconfigured mode */
#if (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Common configuration variables */
    uint8 SCBUART_scbMode = SCBUART_SCB_MODE_UNCONFIG;
    uint8 SCBUART_scbEnableWake;
    uint8 SCBUART_scbEnableIntr;

    /* I2C configuration variables */
    uint8 SCBUART_mode;
    uint8 SCBUART_acceptAddr;

    /* SPI/UART configuration variables */
    volatile uint8 * SCBUART_rxBuffer;
    uint8  SCBUART_rxDataBits;
    uint32 SCBUART_rxBufferSize;

    volatile uint8 * SCBUART_txBuffer;
    uint8  SCBUART_txDataBits;
    uint32 SCBUART_txBufferSize;

    /* EZI2C configuration variables */
    uint8 SCBUART_numberOfAddr;
    uint8 SCBUART_subAddrSize;
#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */


/***************************************
*     Common SCB Vars
***************************************/
/**
* \addtogroup group_general
* \{
*/

/** SCBUART_initVar indicates whether the SCBUART 
*  component has been initialized. The variable is initialized to 0 
*  and set to 1 the first time SCB_Start() is called. This allows 
*  the component to restart without reinitialization after the first 
*  call to the SCBUART_Start() routine.
*
*  If re-initialization of the component is required, then the 
*  SCBUART_Init() function can be called before the 
*  SCBUART_Start() or SCBUART_Enable() function.
*/
uint8 SCBUART_initVar = 0u;


#if (! (SCBUART_SCB_MODE_I2C_CONST_CFG || \
        SCBUART_SCB_MODE_EZI2C_CONST_CFG))
    /** This global variable stores TX interrupt sources after 
    * SCBUART_Stop() is called. Only these TX interrupt sources 
    * will be restored on a subsequent SCBUART_Enable() call.
    */
    uint16 SCBUART_IntrTxMask = 0u;
#endif /* (! (SCBUART_SCB_MODE_I2C_CONST_CFG || \
              SCBUART_SCB_MODE_EZI2C_CONST_CFG)) */
/** \} globals */

#if (SCBUART_SCB_IRQ_INTERNAL)
#if !defined (CY_REMOVE_SCBUART_CUSTOM_INTR_HANDLER)
    void (*SCBUART_customIntrHandler)(void) = NULL;
#endif /* !defined (CY_REMOVE_SCBUART_CUSTOM_INTR_HANDLER) */
#endif /* (SCBUART_SCB_IRQ_INTERNAL) */


/***************************************
*    Private Function Prototypes
***************************************/

static void SCBUART_ScbEnableIntr(void);
static void SCBUART_ScbModeStop(void);
static void SCBUART_ScbModePostEnable(void);


/*******************************************************************************
* Function Name: SCBUART_Init
****************************************************************************//**
*
*  Initializes the SCBUART component to operate in one of the selected
*  configurations: I2C, SPI, UART or EZI2C.
*  When the configuration is set to "Unconfigured SCB", this function does
*  not do any initialization. Use mode-specific initialization APIs instead:
*  SCBUART_I2CInit, SCBUART_SpiInit, 
*  SCBUART_UartInit or SCBUART_EzI2CInit.
*
*******************************************************************************/
void SCBUART_Init(void)
{
#if (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)
    if (SCBUART_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        SCBUART_initVar = 0u;
    }
    else
    {
        /* Initialization was done before this function call */
    }

#elif (SCBUART_SCB_MODE_I2C_CONST_CFG)
    SCBUART_I2CInit();

#elif (SCBUART_SCB_MODE_SPI_CONST_CFG)
    SCBUART_SpiInit();

#elif (SCBUART_SCB_MODE_UART_CONST_CFG)
    SCBUART_UartInit();

#elif (SCBUART_SCB_MODE_EZI2C_CONST_CFG)
    SCBUART_EzI2CInit();

#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SCBUART_Enable
****************************************************************************//**
*
*  Enables SCBUART component operation: activates the hardware and 
*  internal interrupt. It also restores TX interrupt sources disabled after the 
*  SCBUART_Stop() function was called (note that level-triggered TX 
*  interrupt sources remain disabled to not cause code lock-up).
*  For I2C and EZI2C modes the interrupt is internal and mandatory for 
*  operation. For SPI and UART modes the interrupt can be configured as none, 
*  internal or external.
*  The SCBUART configuration should be not changed when the component
*  is enabled. Any configuration changes should be made after disabling the 
*  component.
*  When configuration is set to Unconfigured SCBUART, the component 
*  must first be initialized to operate in one of the following configurations: 
*  I2C, SPI, UART or EZ I2C, using the mode-specific initialization API. 
*  Otherwise this function does not enable the component.
*
*******************************************************************************/
void SCBUART_Enable(void)
{
#if (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Enable SCB block, only if it is already configured */
    if (!SCBUART_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        SCBUART_CTRL_REG |= SCBUART_CTRL_ENABLED;

        SCBUART_ScbEnableIntr();

        /* Call PostEnable function specific to current operation mode */
        SCBUART_ScbModePostEnable();
    }
#else
    SCBUART_CTRL_REG |= SCBUART_CTRL_ENABLED;

    SCBUART_ScbEnableIntr();

    /* Call PostEnable function specific to current operation mode */
    SCBUART_ScbModePostEnable();
#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SCBUART_Start
****************************************************************************//**
*
*  Invokes SCBUART_Init() and SCBUART_Enable().
*  After this function call, the component is enabled and ready for operation.
*  When configuration is set to "Unconfigured SCB", the component must first be
*  initialized to operate in one of the following configurations: I2C, SPI, UART
*  or EZI2C. Otherwise this function does not enable the component.
*
* \globalvars
*  SCBUART_initVar - used to check initial configuration, modified
*  on first function call.
*
*******************************************************************************/
void SCBUART_Start(void)
{
    if (0u == SCBUART_initVar)
    {
        SCBUART_Init();
        SCBUART_initVar = 1u; /* Component was initialized */
    }

    SCBUART_Enable();
}


/*******************************************************************************
* Function Name: SCBUART_Stop
****************************************************************************//**
*
*  Disables the SCBUART component: disable the hardware and internal 
*  interrupt. It also disables all TX interrupt sources so as not to cause an 
*  unexpected interrupt trigger because after the component is enabled, the 
*  TX FIFO is empty.
*  Refer to the function SCBUART_Enable() for the interrupt 
*  configuration details.
*  This function disables the SCB component without checking to see if 
*  communication is in progress. Before calling this function it may be 
*  necessary to check the status of communication to make sure communication 
*  is complete. If this is not done then communication could be stopped mid 
*  byte and corrupted data could result.
*
*******************************************************************************/
void SCBUART_Stop(void)
{
#if (SCBUART_SCB_IRQ_INTERNAL)
    SCBUART_DisableInt();
#endif /* (SCBUART_SCB_IRQ_INTERNAL) */

    /* Call Stop function specific to current operation mode */
    SCBUART_ScbModeStop();

    /* Disable SCB IP */
    SCBUART_CTRL_REG &= (uint32) ~SCBUART_CTRL_ENABLED;

    /* Disable all TX interrupt sources so as not to cause an unexpected
    * interrupt trigger after the component will be enabled because the 
    * TX FIFO is empty.
    * For SCB IP v0, it is critical as it does not mask-out interrupt
    * sources when it is disabled. This can cause a code lock-up in the
    * interrupt handler because TX FIFO cannot be loaded after the block
    * is disabled.
    */
    SCBUART_SetTxInterruptMode(SCBUART_NO_INTR_SOURCES);

#if (SCBUART_SCB_IRQ_INTERNAL)
    SCBUART_ClearPendingInt();
#endif /* (SCBUART_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: SCBUART_SetRxFifoLevel
****************************************************************************//**
*
*  Sets level in the RX FIFO to generate a RX level interrupt.
*  When the RX FIFO has more entries than the RX FIFO level an RX level
*  interrupt request is generated.
*
*  \param level: Level in the RX FIFO to generate RX level interrupt.
*   The range of valid level values is between 0 and RX FIFO depth - 1.
*
*******************************************************************************/
void SCBUART_SetRxFifoLevel(uint32 level)
{
    uint32 rxFifoCtrl;

    rxFifoCtrl = SCBUART_RX_FIFO_CTRL_REG;

    rxFifoCtrl &= ((uint32) ~SCBUART_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    rxFifoCtrl |= ((uint32) (SCBUART_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    SCBUART_RX_FIFO_CTRL_REG = rxFifoCtrl;
}


/*******************************************************************************
* Function Name: SCBUART_SetTxFifoLevel
****************************************************************************//**
*
*  Sets level in the TX FIFO to generate a TX level interrupt.
*  When the TX FIFO has less entries than the TX FIFO level an TX level
*  interrupt request is generated.
*
*  \param level: Level in the TX FIFO to generate TX level interrupt.
*   The range of valid level values is between 0 and TX FIFO depth - 1.
*
*******************************************************************************/
void SCBUART_SetTxFifoLevel(uint32 level)
{
    uint32 txFifoCtrl;

    txFifoCtrl = SCBUART_TX_FIFO_CTRL_REG;

    txFifoCtrl &= ((uint32) ~SCBUART_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    txFifoCtrl |= ((uint32) (SCBUART_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    SCBUART_TX_FIFO_CTRL_REG = txFifoCtrl;
}


#if (SCBUART_SCB_IRQ_INTERNAL)
    /*******************************************************************************
    * Function Name: SCBUART_SetCustomInterruptHandler
    ****************************************************************************//**
    *
    *  Registers a function to be called by the internal interrupt handler.
    *  First the function that is registered is called, then the internal interrupt
    *  handler performs any operation such as software buffer management functions
    *  before the interrupt returns.  It is the user's responsibility not to break
    *  the software buffer operations. Only one custom handler is supported, which
    *  is the function provided by the most recent call.
    *  At the initialization time no custom handler is registered.
    *
    *  \param func: Pointer to the function to register.
    *        The value NULL indicates to remove the current custom interrupt
    *        handler.
    *
    *******************************************************************************/
    void SCBUART_SetCustomInterruptHandler(void (*func)(void))
    {
    #if !defined (CY_REMOVE_SCBUART_CUSTOM_INTR_HANDLER)
        SCBUART_customIntrHandler = func; /* Register interrupt handler */
    #else
        if (NULL != func)
        {
            /* Suppress compiler warning */
        }
    #endif /* !defined (CY_REMOVE_SCBUART_CUSTOM_INTR_HANDLER) */
    }
#endif /* (SCBUART_SCB_IRQ_INTERNAL) */


/*******************************************************************************
* Function Name: SCBUART_ScbModeEnableIntr
****************************************************************************//**
*
*  Enables an interrupt for a specific mode.
*
*******************************************************************************/
static void SCBUART_ScbEnableIntr(void)
{
#if (SCBUART_SCB_IRQ_INTERNAL)
    /* Enable interrupt in NVIC */
    #if (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)
        if (0u != SCBUART_scbEnableIntr)
        {
            SCBUART_EnableInt();
        }

    #else
        SCBUART_EnableInt();

    #endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */
#endif /* (SCBUART_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: SCBUART_ScbModePostEnable
****************************************************************************//**
*
*  Calls the PostEnable function for a specific operation mode.
*
*******************************************************************************/
static void SCBUART_ScbModePostEnable(void)
{
#if (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)
#if (!SCBUART_CY_SCBIP_V1)
    if (SCBUART_SCB_MODE_SPI_RUNTM_CFG)
    {
        SCBUART_SpiPostEnable();
    }
    else if (SCBUART_SCB_MODE_UART_RUNTM_CFG)
    {
        SCBUART_UartPostEnable();
    }
    else
    {
        /* Unknown mode: do nothing */
    }
#endif /* (!SCBUART_CY_SCBIP_V1) */

#elif (SCBUART_SCB_MODE_SPI_CONST_CFG)
    SCBUART_SpiPostEnable();

#elif (SCBUART_SCB_MODE_UART_CONST_CFG)
    SCBUART_UartPostEnable();

#else
    /* Unknown mode: do nothing */
#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SCBUART_ScbModeStop
****************************************************************************//**
*
*  Calls the Stop function for a specific operation mode.
*
*******************************************************************************/
static void SCBUART_ScbModeStop(void)
{
#if (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)
    if (SCBUART_SCB_MODE_I2C_RUNTM_CFG)
    {
        SCBUART_I2CStop();
    }
    else if (SCBUART_SCB_MODE_EZI2C_RUNTM_CFG)
    {
        SCBUART_EzI2CStop();
    }
#if (!SCBUART_CY_SCBIP_V1)
    else if (SCBUART_SCB_MODE_SPI_RUNTM_CFG)
    {
        SCBUART_SpiStop();
    }
    else if (SCBUART_SCB_MODE_UART_RUNTM_CFG)
    {
        SCBUART_UartStop();
    }
#endif /* (!SCBUART_CY_SCBIP_V1) */
    else
    {
        /* Unknown mode: do nothing */
    }
#elif (SCBUART_SCB_MODE_I2C_CONST_CFG)
    SCBUART_I2CStop();

#elif (SCBUART_SCB_MODE_EZI2C_CONST_CFG)
    SCBUART_EzI2CStop();

#elif (SCBUART_SCB_MODE_SPI_CONST_CFG)
    SCBUART_SpiStop();

#elif (SCBUART_SCB_MODE_UART_CONST_CFG)
    SCBUART_UartStop();

#else
    /* Unknown mode: do nothing */
#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */
}


#if (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: SCBUART_SetPins
    ****************************************************************************//**
    *
    *  Sets the pins settings accordingly to the selected operation mode.
    *  Only available in the Unconfigured operation mode. The mode specific
    *  initialization function calls it.
    *  Pins configuration is set by PSoC Creator when a specific mode of operation
    *  is selected in design time.
    *
    *  \param mode:      Mode of SCB operation.
    *  \param subMode:   Sub-mode of SCB operation. It is only required for SPI and UART
    *             modes.
    *  \param uartEnableMask: enables TX or RX direction and RTS and CTS signals.
    *
    *******************************************************************************/
    void SCBUART_SetPins(uint32 mode, uint32 subMode, uint32 uartEnableMask)
    {
        uint32 pinsDm[SCBUART_SCB_PINS_NUMBER];
        uint32 i;
        
    #if (!SCBUART_CY_SCBIP_V1)
        uint32 pinsInBuf = 0u;
    #endif /* (!SCBUART_CY_SCBIP_V1) */
        
        uint32 hsiomSel[SCBUART_SCB_PINS_NUMBER] = 
        {
            SCBUART_RX_SCL_MOSI_HSIOM_SEL_GPIO,
            SCBUART_TX_SDA_MISO_HSIOM_SEL_GPIO,
            0u,
            0u,
            0u,
            0u,
            0u,
        };

    #if (SCBUART_CY_SCBIP_V1)
        /* Supress compiler warning. */
        if ((0u == subMode) || (0u == uartEnableMask))
        {
        }
    #endif /* (SCBUART_CY_SCBIP_V1) */

        /* Set default HSIOM to GPIO and Drive Mode to Analog Hi-Z */
        for (i = 0u; i < SCBUART_SCB_PINS_NUMBER; i++)
        {
            pinsDm[i] = SCBUART_PIN_DM_ALG_HIZ;
        }

        if ((SCBUART_SCB_MODE_I2C   == mode) ||
            (SCBUART_SCB_MODE_EZI2C == mode))
        {
        #if (SCBUART_RX_SCL_MOSI_PIN)
            hsiomSel[SCBUART_RX_SCL_MOSI_PIN_INDEX] = SCBUART_RX_SCL_MOSI_HSIOM_SEL_I2C;
            pinsDm  [SCBUART_RX_SCL_MOSI_PIN_INDEX] = SCBUART_PIN_DM_OD_LO;
        #elif (SCBUART_RX_WAKE_SCL_MOSI_PIN)
            hsiomSel[SCBUART_RX_WAKE_SCL_MOSI_PIN_INDEX] = SCBUART_RX_WAKE_SCL_MOSI_HSIOM_SEL_I2C;
            pinsDm  [SCBUART_RX_WAKE_SCL_MOSI_PIN_INDEX] = SCBUART_PIN_DM_OD_LO;
        #else
        #endif /* (SCBUART_RX_SCL_MOSI_PIN) */
        
        #if (SCBUART_TX_SDA_MISO_PIN)
            hsiomSel[SCBUART_TX_SDA_MISO_PIN_INDEX] = SCBUART_TX_SDA_MISO_HSIOM_SEL_I2C;
            pinsDm  [SCBUART_TX_SDA_MISO_PIN_INDEX] = SCBUART_PIN_DM_OD_LO;
        #endif /* (SCBUART_TX_SDA_MISO_PIN) */
        }
    #if (!SCBUART_CY_SCBIP_V1)
        else if (SCBUART_SCB_MODE_SPI == mode)
        {
        #if (SCBUART_RX_SCL_MOSI_PIN)
            hsiomSel[SCBUART_RX_SCL_MOSI_PIN_INDEX] = SCBUART_RX_SCL_MOSI_HSIOM_SEL_SPI;
        #elif (SCBUART_RX_WAKE_SCL_MOSI_PIN)
            hsiomSel[SCBUART_RX_WAKE_SCL_MOSI_PIN_INDEX] = SCBUART_RX_WAKE_SCL_MOSI_HSIOM_SEL_SPI;
        #else
        #endif /* (SCBUART_RX_SCL_MOSI_PIN) */
        
        #if (SCBUART_TX_SDA_MISO_PIN)
            hsiomSel[SCBUART_TX_SDA_MISO_PIN_INDEX] = SCBUART_TX_SDA_MISO_HSIOM_SEL_SPI;
        #endif /* (SCBUART_TX_SDA_MISO_PIN) */
        
        #if (SCBUART_CTS_SCLK_PIN)
            hsiomSel[SCBUART_CTS_SCLK_PIN_INDEX] = SCBUART_CTS_SCLK_HSIOM_SEL_SPI;
        #endif /* (SCBUART_CTS_SCLK_PIN) */

            if (SCBUART_SPI_SLAVE == subMode)
            {
                /* Slave */
                pinsDm[SCBUART_RX_SCL_MOSI_PIN_INDEX] = SCBUART_PIN_DM_DIG_HIZ;
                pinsDm[SCBUART_TX_SDA_MISO_PIN_INDEX] = SCBUART_PIN_DM_STRONG;
                pinsDm[SCBUART_CTS_SCLK_PIN_INDEX] = SCBUART_PIN_DM_DIG_HIZ;

            #if (SCBUART_RTS_SS0_PIN)
                /* Only SS0 is valid choice for Slave */
                hsiomSel[SCBUART_RTS_SS0_PIN_INDEX] = SCBUART_RTS_SS0_HSIOM_SEL_SPI;
                pinsDm  [SCBUART_RTS_SS0_PIN_INDEX] = SCBUART_PIN_DM_DIG_HIZ;
            #endif /* (SCBUART_RTS_SS0_PIN) */

            #if (SCBUART_TX_SDA_MISO_PIN)
                /* Disable input buffer */
                 pinsInBuf |= SCBUART_TX_SDA_MISO_PIN_MASK;
            #endif /* (SCBUART_TX_SDA_MISO_PIN) */
            }
            else 
            {
                /* (Master) */
                pinsDm[SCBUART_RX_SCL_MOSI_PIN_INDEX] = SCBUART_PIN_DM_STRONG;
                pinsDm[SCBUART_TX_SDA_MISO_PIN_INDEX] = SCBUART_PIN_DM_DIG_HIZ;
                pinsDm[SCBUART_CTS_SCLK_PIN_INDEX] = SCBUART_PIN_DM_STRONG;

            #if (SCBUART_RTS_SS0_PIN)
                hsiomSel [SCBUART_RTS_SS0_PIN_INDEX] = SCBUART_RTS_SS0_HSIOM_SEL_SPI;
                pinsDm   [SCBUART_RTS_SS0_PIN_INDEX] = SCBUART_PIN_DM_STRONG;
                pinsInBuf |= SCBUART_RTS_SS0_PIN_MASK;
            #endif /* (SCBUART_RTS_SS0_PIN) */

            #if (SCBUART_SS1_PIN)
                hsiomSel [SCBUART_SS1_PIN_INDEX] = SCBUART_SS1_HSIOM_SEL_SPI;
                pinsDm   [SCBUART_SS1_PIN_INDEX] = SCBUART_PIN_DM_STRONG;
                pinsInBuf |= SCBUART_SS1_PIN_MASK;
            #endif /* (SCBUART_SS1_PIN) */

            #if (SCBUART_SS2_PIN)
                hsiomSel [SCBUART_SS2_PIN_INDEX] = SCBUART_SS2_HSIOM_SEL_SPI;
                pinsDm   [SCBUART_SS2_PIN_INDEX] = SCBUART_PIN_DM_STRONG;
                pinsInBuf |= SCBUART_SS2_PIN_MASK;
            #endif /* (SCBUART_SS2_PIN) */

            #if (SCBUART_SS3_PIN)
                hsiomSel [SCBUART_SS3_PIN_INDEX] = SCBUART_SS3_HSIOM_SEL_SPI;
                pinsDm   [SCBUART_SS3_PIN_INDEX] = SCBUART_PIN_DM_STRONG;
                pinsInBuf |= SCBUART_SS3_PIN_MASK;
            #endif /* (SCBUART_SS3_PIN) */

                /* Disable input buffers */
            #if (SCBUART_RX_SCL_MOSI_PIN)
                pinsInBuf |= SCBUART_RX_SCL_MOSI_PIN_MASK;
            #elif (SCBUART_RX_WAKE_SCL_MOSI_PIN)
                pinsInBuf |= SCBUART_RX_WAKE_SCL_MOSI_PIN_MASK;
            #else
            #endif /* (SCBUART_RX_SCL_MOSI_PIN) */

            #if (SCBUART_CTS_SCLK_PIN)
                pinsInBuf |= SCBUART_CTS_SCLK_PIN_MASK;
            #endif /* (SCBUART_CTS_SCLK_PIN) */
            }
        }
        else /* UART */
        {
            if (SCBUART_UART_MODE_SMARTCARD == subMode)
            {
                /* SmartCard */
            #if (SCBUART_TX_SDA_MISO_PIN)
                hsiomSel[SCBUART_TX_SDA_MISO_PIN_INDEX] = SCBUART_TX_SDA_MISO_HSIOM_SEL_UART;
                pinsDm  [SCBUART_TX_SDA_MISO_PIN_INDEX] = SCBUART_PIN_DM_OD_LO;
            #endif /* (SCBUART_TX_SDA_MISO_PIN) */
            }
            else /* Standard or IrDA */
            {
                if (0u != (SCBUART_UART_RX_PIN_ENABLE & uartEnableMask))
                {
                #if (SCBUART_RX_SCL_MOSI_PIN)
                    hsiomSel[SCBUART_RX_SCL_MOSI_PIN_INDEX] = SCBUART_RX_SCL_MOSI_HSIOM_SEL_UART;
                    pinsDm  [SCBUART_RX_SCL_MOSI_PIN_INDEX] = SCBUART_PIN_DM_DIG_HIZ;
                #elif (SCBUART_RX_WAKE_SCL_MOSI_PIN)
                    hsiomSel[SCBUART_RX_WAKE_SCL_MOSI_PIN_INDEX] = SCBUART_RX_WAKE_SCL_MOSI_HSIOM_SEL_UART;
                    pinsDm  [SCBUART_RX_WAKE_SCL_MOSI_PIN_INDEX] = SCBUART_PIN_DM_DIG_HIZ;
                #else
                #endif /* (SCBUART_RX_SCL_MOSI_PIN) */
                }

                if (0u != (SCBUART_UART_TX_PIN_ENABLE & uartEnableMask))
                {
                #if (SCBUART_TX_SDA_MISO_PIN)
                    hsiomSel[SCBUART_TX_SDA_MISO_PIN_INDEX] = SCBUART_TX_SDA_MISO_HSIOM_SEL_UART;
                    pinsDm  [SCBUART_TX_SDA_MISO_PIN_INDEX] = SCBUART_PIN_DM_STRONG;
                    
                    /* Disable input buffer */
                    pinsInBuf |= SCBUART_TX_SDA_MISO_PIN_MASK;
                #endif /* (SCBUART_TX_SDA_MISO_PIN) */
                }

            #if !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1)
                if (SCBUART_UART_MODE_STD == subMode)
                {
                    if (0u != (SCBUART_UART_CTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* CTS input is multiplexed with SCLK */
                    #if (SCBUART_CTS_SCLK_PIN)
                        hsiomSel[SCBUART_CTS_SCLK_PIN_INDEX] = SCBUART_CTS_SCLK_HSIOM_SEL_UART;
                        pinsDm  [SCBUART_CTS_SCLK_PIN_INDEX] = SCBUART_PIN_DM_DIG_HIZ;
                    #endif /* (SCBUART_CTS_SCLK_PIN) */
                    }

                    if (0u != (SCBUART_UART_RTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* RTS output is multiplexed with SS0 */
                    #if (SCBUART_RTS_SS0_PIN)
                        hsiomSel[SCBUART_RTS_SS0_PIN_INDEX] = SCBUART_RTS_SS0_HSIOM_SEL_UART;
                        pinsDm  [SCBUART_RTS_SS0_PIN_INDEX] = SCBUART_PIN_DM_STRONG;
                        
                        /* Disable input buffer */
                        pinsInBuf |= SCBUART_RTS_SS0_PIN_MASK;
                    #endif /* (SCBUART_RTS_SS0_PIN) */
                    }
                }
            #endif /* !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1) */
            }
        }
    #endif /* (!SCBUART_CY_SCBIP_V1) */

    /* Configure pins: set HSIOM, DM and InputBufEnable */
    /* Note: the DR register settings do not effect the pin output if HSIOM is other than GPIO */

    #if (SCBUART_RX_SCL_MOSI_PIN)
        SCBUART_SET_HSIOM_SEL(SCBUART_RX_SCL_MOSI_HSIOM_REG,
                                       SCBUART_RX_SCL_MOSI_HSIOM_MASK,
                                       SCBUART_RX_SCL_MOSI_HSIOM_POS,
                                        hsiomSel[SCBUART_RX_SCL_MOSI_PIN_INDEX]);

        SCBUART_uart_rx_i2c_scl_spi_mosi_SetDriveMode((uint8) pinsDm[SCBUART_RX_SCL_MOSI_PIN_INDEX]);

        #if (!SCBUART_CY_SCBIP_V1)
            SCBUART_SET_INP_DIS(SCBUART_uart_rx_i2c_scl_spi_mosi_INP_DIS,
                                         SCBUART_uart_rx_i2c_scl_spi_mosi_MASK,
                                         (0u != (pinsInBuf & SCBUART_RX_SCL_MOSI_PIN_MASK)));
        #endif /* (!SCBUART_CY_SCBIP_V1) */
    
    #elif (SCBUART_RX_WAKE_SCL_MOSI_PIN)
        SCBUART_SET_HSIOM_SEL(SCBUART_RX_WAKE_SCL_MOSI_HSIOM_REG,
                                       SCBUART_RX_WAKE_SCL_MOSI_HSIOM_MASK,
                                       SCBUART_RX_WAKE_SCL_MOSI_HSIOM_POS,
                                       hsiomSel[SCBUART_RX_WAKE_SCL_MOSI_PIN_INDEX]);

        SCBUART_uart_rx_wake_i2c_scl_spi_mosi_SetDriveMode((uint8)
                                                               pinsDm[SCBUART_RX_WAKE_SCL_MOSI_PIN_INDEX]);

        SCBUART_SET_INP_DIS(SCBUART_uart_rx_wake_i2c_scl_spi_mosi_INP_DIS,
                                     SCBUART_uart_rx_wake_i2c_scl_spi_mosi_MASK,
                                     (0u != (pinsInBuf & SCBUART_RX_WAKE_SCL_MOSI_PIN_MASK)));

         /* Set interrupt on falling edge */
        SCBUART_SET_INCFG_TYPE(SCBUART_RX_WAKE_SCL_MOSI_INTCFG_REG,
                                        SCBUART_RX_WAKE_SCL_MOSI_INTCFG_TYPE_MASK,
                                        SCBUART_RX_WAKE_SCL_MOSI_INTCFG_TYPE_POS,
                                        SCBUART_INTCFG_TYPE_FALLING_EDGE);
    #else
    #endif /* (SCBUART_RX_WAKE_SCL_MOSI_PIN) */

    #if (SCBUART_TX_SDA_MISO_PIN)
        SCBUART_SET_HSIOM_SEL(SCBUART_TX_SDA_MISO_HSIOM_REG,
                                       SCBUART_TX_SDA_MISO_HSIOM_MASK,
                                       SCBUART_TX_SDA_MISO_HSIOM_POS,
                                        hsiomSel[SCBUART_TX_SDA_MISO_PIN_INDEX]);

        SCBUART_uart_tx_i2c_sda_spi_miso_SetDriveMode((uint8) pinsDm[SCBUART_TX_SDA_MISO_PIN_INDEX]);

    #if (!SCBUART_CY_SCBIP_V1)
        SCBUART_SET_INP_DIS(SCBUART_uart_tx_i2c_sda_spi_miso_INP_DIS,
                                     SCBUART_uart_tx_i2c_sda_spi_miso_MASK,
                                    (0u != (pinsInBuf & SCBUART_TX_SDA_MISO_PIN_MASK)));
    #endif /* (!SCBUART_CY_SCBIP_V1) */
    #endif /* (SCBUART_RX_SCL_MOSI_PIN) */

    #if (SCBUART_CTS_SCLK_PIN)
        SCBUART_SET_HSIOM_SEL(SCBUART_CTS_SCLK_HSIOM_REG,
                                       SCBUART_CTS_SCLK_HSIOM_MASK,
                                       SCBUART_CTS_SCLK_HSIOM_POS,
                                       hsiomSel[SCBUART_CTS_SCLK_PIN_INDEX]);

        SCBUART_uart_cts_spi_sclk_SetDriveMode((uint8) pinsDm[SCBUART_CTS_SCLK_PIN_INDEX]);

        SCBUART_SET_INP_DIS(SCBUART_uart_cts_spi_sclk_INP_DIS,
                                     SCBUART_uart_cts_spi_sclk_MASK,
                                     (0u != (pinsInBuf & SCBUART_CTS_SCLK_PIN_MASK)));
    #endif /* (SCBUART_CTS_SCLK_PIN) */

    #if (SCBUART_RTS_SS0_PIN)
        SCBUART_SET_HSIOM_SEL(SCBUART_RTS_SS0_HSIOM_REG,
                                       SCBUART_RTS_SS0_HSIOM_MASK,
                                       SCBUART_RTS_SS0_HSIOM_POS,
                                       hsiomSel[SCBUART_RTS_SS0_PIN_INDEX]);

        SCBUART_uart_rts_spi_ss0_SetDriveMode((uint8) pinsDm[SCBUART_RTS_SS0_PIN_INDEX]);

        SCBUART_SET_INP_DIS(SCBUART_uart_rts_spi_ss0_INP_DIS,
                                     SCBUART_uart_rts_spi_ss0_MASK,
                                     (0u != (pinsInBuf & SCBUART_RTS_SS0_PIN_MASK)));
    #endif /* (SCBUART_RTS_SS0_PIN) */

    #if (SCBUART_SS1_PIN)
        SCBUART_SET_HSIOM_SEL(SCBUART_SS1_HSIOM_REG,
                                       SCBUART_SS1_HSIOM_MASK,
                                       SCBUART_SS1_HSIOM_POS,
                                       hsiomSel[SCBUART_SS1_PIN_INDEX]);

        SCBUART_spi_ss1_SetDriveMode((uint8) pinsDm[SCBUART_SS1_PIN_INDEX]);

        SCBUART_SET_INP_DIS(SCBUART_spi_ss1_INP_DIS,
                                     SCBUART_spi_ss1_MASK,
                                     (0u != (pinsInBuf & SCBUART_SS1_PIN_MASK)));
    #endif /* (SCBUART_SS1_PIN) */

    #if (SCBUART_SS2_PIN)
        SCBUART_SET_HSIOM_SEL(SCBUART_SS2_HSIOM_REG,
                                       SCBUART_SS2_HSIOM_MASK,
                                       SCBUART_SS2_HSIOM_POS,
                                       hsiomSel[SCBUART_SS2_PIN_INDEX]);

        SCBUART_spi_ss2_SetDriveMode((uint8) pinsDm[SCBUART_SS2_PIN_INDEX]);

        SCBUART_SET_INP_DIS(SCBUART_spi_ss2_INP_DIS,
                                     SCBUART_spi_ss2_MASK,
                                     (0u != (pinsInBuf & SCBUART_SS2_PIN_MASK)));
    #endif /* (SCBUART_SS2_PIN) */

    #if (SCBUART_SS3_PIN)
        SCBUART_SET_HSIOM_SEL(SCBUART_SS3_HSIOM_REG,
                                       SCBUART_SS3_HSIOM_MASK,
                                       SCBUART_SS3_HSIOM_POS,
                                       hsiomSel[SCBUART_SS3_PIN_INDEX]);

        SCBUART_spi_ss3_SetDriveMode((uint8) pinsDm[SCBUART_SS3_PIN_INDEX]);

        SCBUART_SET_INP_DIS(SCBUART_spi_ss3_INP_DIS,
                                     SCBUART_spi_ss3_MASK,
                                     (0u != (pinsInBuf & SCBUART_SS3_PIN_MASK)));
    #endif /* (SCBUART_SS3_PIN) */
    }

#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */


#if (SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1)
    /*******************************************************************************
    * Function Name: SCBUART_I2CSlaveNackGeneration
    ****************************************************************************//**
    *
    *  Sets command to generate NACK to the address or data.
    *
    *******************************************************************************/
    void SCBUART_I2CSlaveNackGeneration(void)
    {
        /* Check for EC_AM toggle condition: EC_AM and clock stretching for address are enabled */
        if ((0u != (SCBUART_CTRL_REG & SCBUART_CTRL_EC_AM_MODE)) &&
            (0u == (SCBUART_I2C_CTRL_REG & SCBUART_I2C_CTRL_S_NOT_READY_ADDR_NACK)))
        {
            /* Toggle EC_AM before NACK generation */
            SCBUART_CTRL_REG &= ~SCBUART_CTRL_EC_AM_MODE;
            SCBUART_CTRL_REG |=  SCBUART_CTRL_EC_AM_MODE;
        }

        SCBUART_I2C_SLAVE_CMD_REG = SCBUART_I2C_SLAVE_CMD_S_NACK;
    }
#endif /* (SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1) */


/* [] END OF FILE */
