/***************************************************************************//**
* \file SCBUART_PM.c
* \version 3.20
*
* \brief
*  This file provides the source code to the Power Management support for
*  the SCB Component.
*
* Note:
*
********************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SCBUART.h"
#include "SCBUART_PVT.h"

#if(SCBUART_SCB_MODE_I2C_INC)
    #include "SCBUART_I2C_PVT.h"
#endif /* (SCBUART_SCB_MODE_I2C_INC) */

#if(SCBUART_SCB_MODE_EZI2C_INC)
    #include "SCBUART_EZI2C_PVT.h"
#endif /* (SCBUART_SCB_MODE_EZI2C_INC) */

#if(SCBUART_SCB_MODE_SPI_INC || SCBUART_SCB_MODE_UART_INC)
    #include "SCBUART_SPI_UART_PVT.h"
#endif /* (SCBUART_SCB_MODE_SPI_INC || SCBUART_SCB_MODE_UART_INC) */


/***************************************
*   Backup Structure declaration
***************************************/

#if(SCBUART_SCB_MODE_UNCONFIG_CONST_CFG || \
   (SCBUART_SCB_MODE_I2C_CONST_CFG   && (!SCBUART_I2C_WAKE_ENABLE_CONST))   || \
   (SCBUART_SCB_MODE_EZI2C_CONST_CFG && (!SCBUART_EZI2C_WAKE_ENABLE_CONST)) || \
   (SCBUART_SCB_MODE_SPI_CONST_CFG   && (!SCBUART_SPI_WAKE_ENABLE_CONST))   || \
   (SCBUART_SCB_MODE_UART_CONST_CFG  && (!SCBUART_UART_WAKE_ENABLE_CONST)))

    SCBUART_BACKUP_STRUCT SCBUART_backup =
    {
        0u, /* enableState */
    };
#endif


/*******************************************************************************
* Function Name: SCBUART_Sleep
****************************************************************************//**
*
*  Prepares the SCBUART component to enter Deep Sleep.
*  The Enable wakeup from Deep Sleep Mode selection has an influence on this 
*  function implementation:
*  - Checked: configures the component to be wakeup source from Deep Sleep.
*  - Unchecked: stores the current component state (enabled or disabled) and 
*    disables the component. See SCB_Stop() function for details about component 
*    disabling.
*
*  Call the SCBUART_Sleep() function before calling the 
*  CyPmSysDeepSleep() function. 
*  Refer to the PSoC Creator System Reference Guide for more information about 
*  power management functions and Low power section of this document for the 
*  selected mode.
*
*  This function should not be called before entering Sleep.
*
*******************************************************************************/
void SCBUART_Sleep(void)
{
#if(SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)

    if(SCBUART_SCB_WAKE_ENABLE_CHECK)
    {
        if(SCBUART_SCB_MODE_I2C_RUNTM_CFG)
        {
            SCBUART_I2CSaveConfig();
        }
        else if(SCBUART_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            SCBUART_EzI2CSaveConfig();
        }
    #if(!SCBUART_CY_SCBIP_V1)
        else if(SCBUART_SCB_MODE_SPI_RUNTM_CFG)
        {
            SCBUART_SpiSaveConfig();
        }
        else if(SCBUART_SCB_MODE_UART_RUNTM_CFG)
        {
            SCBUART_UartSaveConfig();
        }
    #endif /* (!SCBUART_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        SCBUART_backup.enableState = (uint8) SCBUART_GET_CTRL_ENABLED;

        if(0u != SCBUART_backup.enableState)
        {
            SCBUART_Stop();
        }
    }

#else

    #if (SCBUART_SCB_MODE_I2C_CONST_CFG && SCBUART_I2C_WAKE_ENABLE_CONST)
        SCBUART_I2CSaveConfig();

    #elif (SCBUART_SCB_MODE_EZI2C_CONST_CFG && SCBUART_EZI2C_WAKE_ENABLE_CONST)
        SCBUART_EzI2CSaveConfig();

    #elif (SCBUART_SCB_MODE_SPI_CONST_CFG && SCBUART_SPI_WAKE_ENABLE_CONST)
        SCBUART_SpiSaveConfig();

    #elif (SCBUART_SCB_MODE_UART_CONST_CFG && SCBUART_UART_WAKE_ENABLE_CONST)
        SCBUART_UartSaveConfig();

    #else

        SCBUART_backup.enableState = (uint8) SCBUART_GET_CTRL_ENABLED;

        if(0u != SCBUART_backup.enableState)
        {
            SCBUART_Stop();
        }

    #endif /* defined (SCBUART_SCB_MODE_I2C_CONST_CFG) && (SCBUART_I2C_WAKE_ENABLE_CONST) */

#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SCBUART_Wakeup
****************************************************************************//**
*
*  Prepares the SCBUART component for Active mode operation after 
*  Deep Sleep.
*  The Enable wakeup from Deep Sleep Mode selection has influence on this 
*  function implementation:
*  - Checked: restores the component Active mode configuration.
*  - Unchecked: enables the component if it was enabled before enter Deep Sleep.
*
*  This function should not be called after exiting Sleep.
*
*  \sideeffect
*   Calling the SCBUART_Wakeup() function without first calling the 
*   SCBUART_Sleep() function may produce unexpected behavior.
*
*******************************************************************************/
void SCBUART_Wakeup(void)
{
#if(SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)

    if(SCBUART_SCB_WAKE_ENABLE_CHECK)
    {
        if(SCBUART_SCB_MODE_I2C_RUNTM_CFG)
        {
            SCBUART_I2CRestoreConfig();
        }
        else if(SCBUART_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            SCBUART_EzI2CRestoreConfig();
        }
    #if(!SCBUART_CY_SCBIP_V1)
        else if(SCBUART_SCB_MODE_SPI_RUNTM_CFG)
        {
            SCBUART_SpiRestoreConfig();
        }
        else if(SCBUART_SCB_MODE_UART_RUNTM_CFG)
        {
            SCBUART_UartRestoreConfig();
        }
    #endif /* (!SCBUART_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        if(0u != SCBUART_backup.enableState)
        {
            SCBUART_Enable();
        }
    }

#else

    #if (SCBUART_SCB_MODE_I2C_CONST_CFG  && SCBUART_I2C_WAKE_ENABLE_CONST)
        SCBUART_I2CRestoreConfig();

    #elif (SCBUART_SCB_MODE_EZI2C_CONST_CFG && SCBUART_EZI2C_WAKE_ENABLE_CONST)
        SCBUART_EzI2CRestoreConfig();

    #elif (SCBUART_SCB_MODE_SPI_CONST_CFG && SCBUART_SPI_WAKE_ENABLE_CONST)
        SCBUART_SpiRestoreConfig();

    #elif (SCBUART_SCB_MODE_UART_CONST_CFG && SCBUART_UART_WAKE_ENABLE_CONST)
        SCBUART_UartRestoreConfig();

    #else

        if(0u != SCBUART_backup.enableState)
        {
            SCBUART_Enable();
        }

    #endif /* (SCBUART_I2C_WAKE_ENABLE_CONST) */

#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/* [] END OF FILE */
