/***************************************************************************//**
* \file SCBUART_SPI_UART.c
* \version 3.20
*
* \brief
*  This file provides the source code to the API for the SCB Component in
*  SPI and UART modes.
*
* Note:
*
*******************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SCBUART_PVT.h"
#include "SCBUART_SPI_UART_PVT.h"

/***************************************
*        SPI/UART Private Vars
***************************************/

#if(SCBUART_INTERNAL_RX_SW_BUFFER_CONST)
    /* Start index to put data into the software receive buffer.*/
    volatile uint32 SCBUART_rxBufferHead;
    /* Start index to get data from the software receive buffer.*/
    volatile uint32 SCBUART_rxBufferTail;
    /**
    * \addtogroup group_globals
    * \{
    */
    /** Sets when internal software receive buffer overflow
    *  was occurred.
    */
    volatile uint8  SCBUART_rxBufferOverflow;
    /** \} globals */
#endif /* (SCBUART_INTERNAL_RX_SW_BUFFER_CONST) */

#if(SCBUART_INTERNAL_TX_SW_BUFFER_CONST)
    /* Start index to put data into the software transmit buffer.*/
    volatile uint32 SCBUART_txBufferHead;
    /* Start index to get data from the software transmit buffer.*/
    volatile uint32 SCBUART_txBufferTail;
#endif /* (SCBUART_INTERNAL_TX_SW_BUFFER_CONST) */

#if(SCBUART_INTERNAL_RX_SW_BUFFER)
    /* Add one element to the buffer to receive full packet. One byte in receive buffer is always empty */
    volatile uint8 SCBUART_rxBufferInternal[SCBUART_INTERNAL_RX_BUFFER_SIZE];
#endif /* (SCBUART_INTERNAL_RX_SW_BUFFER) */

#if(SCBUART_INTERNAL_TX_SW_BUFFER)
    volatile uint8 SCBUART_txBufferInternal[SCBUART_TX_BUFFER_SIZE];
#endif /* (SCBUART_INTERNAL_TX_SW_BUFFER) */


#if(SCBUART_RX_DIRECTION)
    /*******************************************************************************
    * Function Name: SCBUART_SpiUartReadRxData
    ****************************************************************************//**
    *
    *  Retrieves the next data element from the receive buffer.
    *   - RX software buffer is disabled: Returns data element retrieved from
    *     RX FIFO. Undefined data will be returned if the RX FIFO is empty.
    *   - RX software buffer is enabled: Returns data element from the software
    *     receive buffer. Zero value is returned if the software receive buffer
    *     is empty.
    *
    * \return
    *  Next data element from the receive buffer. 
    *  The amount of data bits to be received depends on RX data bits selection 
    *  (the data bit counting starts from LSB of return value).
    *
    * \globalvars
    *  SCBUART_rxBufferHead - the start index to put data into the 
    *  software receive buffer.
    *  SCBUART_rxBufferTail - the start index to get data from the 
    *  software receive buffer.
    *
    *******************************************************************************/
    uint32 SCBUART_SpiUartReadRxData(void)
    {
        uint32 rxData = 0u;

    #if (SCBUART_INTERNAL_RX_SW_BUFFER_CONST)
        uint32 locTail;
    #endif /* (SCBUART_INTERNAL_RX_SW_BUFFER_CONST) */

        #if (SCBUART_CHECK_RX_SW_BUFFER)
        {
            if (SCBUART_rxBufferHead != SCBUART_rxBufferTail)
            {
                /* There is data in RX software buffer */

                /* Calculate index to read from */
                locTail = (SCBUART_rxBufferTail + 1u);

                if (SCBUART_INTERNAL_RX_BUFFER_SIZE == locTail)
                {
                    locTail = 0u;
                }

                /* Get data from RX software buffer */
                rxData = SCBUART_GetWordFromRxBuffer(locTail);

                /* Change index in the buffer */
                SCBUART_rxBufferTail = locTail;

                #if (SCBUART_CHECK_UART_RTS_CONTROL_FLOW)
                {
                    /* Check if RX Not Empty is disabled in the interrupt */
                    if (0u == (SCBUART_INTR_RX_MASK_REG & SCBUART_INTR_RX_NOT_EMPTY))
                    {
                        /* Enable RX Not Empty interrupt source to continue
                        * receiving data into software buffer.
                        */
                        SCBUART_INTR_RX_MASK_REG |= SCBUART_INTR_RX_NOT_EMPTY;
                    }
                }
                #endif

            }
        }
        #else
        {
            /* Read data from RX FIFO */
            rxData = SCBUART_RX_FIFO_RD_REG;
        }
        #endif

        return (rxData);
    }


    /*******************************************************************************
    * Function Name: SCBUART_SpiUartGetRxBufferSize
    ****************************************************************************//**
    *
    *  Returns the number of received data elements in the receive buffer.
    *   - RX software buffer disabled: returns the number of used entries in
    *     RX FIFO.
    *   - RX software buffer enabled: returns the number of elements which were
    *     placed in the receive buffer. This does not include the hardware RX FIFO.
    *
    * \return
    *  Number of received data elements.
    *
    * \globalvars
    *  SCBUART_rxBufferHead - the start index to put data into the 
    *  software receive buffer.
    *  SCBUART_rxBufferTail - the start index to get data from the 
    *  software receive buffer.
    *
    *******************************************************************************/
    uint32 SCBUART_SpiUartGetRxBufferSize(void)
    {
        uint32 size;
    #if (SCBUART_INTERNAL_RX_SW_BUFFER_CONST)
        uint32 locHead;
    #endif /* (SCBUART_INTERNAL_RX_SW_BUFFER_CONST) */

        #if (SCBUART_CHECK_RX_SW_BUFFER)
        {
            locHead = SCBUART_rxBufferHead;

            if(locHead >= SCBUART_rxBufferTail)
            {
                size = (locHead - SCBUART_rxBufferTail);
            }
            else
            {
                size = (locHead + (SCBUART_INTERNAL_RX_BUFFER_SIZE - SCBUART_rxBufferTail));
            }
        }
        #else
        {
            size = SCBUART_GET_RX_FIFO_ENTRIES;
        }
        #endif

        return (size);
    }


    /*******************************************************************************
    * Function Name: SCBUART_SpiUartClearRxBuffer
    ****************************************************************************//**
    *
    *  Clears the receive buffer and RX FIFO.
    *
    * \globalvars
    *  SCBUART_rxBufferHead - the start index to put data into the 
    *  software receive buffer.
    *  SCBUART_rxBufferTail - the start index to get data from the 
    *  software receive buffer.
    *
    *******************************************************************************/
    void SCBUART_SpiUartClearRxBuffer(void)
    {
        #if (SCBUART_CHECK_RX_SW_BUFFER)
        {
            /* Lock from component interruption */
            SCBUART_DisableInt();

            /* Flush RX software buffer */
            SCBUART_rxBufferHead = SCBUART_rxBufferTail;
            SCBUART_rxBufferOverflow = 0u;

            SCBUART_CLEAR_RX_FIFO;
            SCBUART_ClearRxInterruptSource(SCBUART_INTR_RX_ALL);

            #if (SCBUART_CHECK_UART_RTS_CONTROL_FLOW)
            {
                /* Enable RX Not Empty interrupt source to continue receiving
                * data into software buffer.
                */
                SCBUART_INTR_RX_MASK_REG |= SCBUART_INTR_RX_NOT_EMPTY;
            }
            #endif
            
            /* Release lock */
            SCBUART_EnableInt();
        }
        #else
        {
            SCBUART_CLEAR_RX_FIFO;
        }
        #endif
    }

#endif /* (SCBUART_RX_DIRECTION) */


#if(SCBUART_TX_DIRECTION)
    /*******************************************************************************
    * Function Name: SCBUART_SpiUartWriteTxData
    ****************************************************************************//**
    *
    *  Places a data entry into the transmit buffer to be sent at the next available
    *  bus time.
    *  This function is blocking and waits until there is space available to put the
    *  requested data in the transmit buffer.
    *
    *  \param txDataByte: the data to be transmitted.
    *   The amount of data bits to be transmitted depends on TX data bits selection 
    *   (the data bit counting starts from LSB of txDataByte).
    *
    * \globalvars
    *  SCBUART_txBufferHead - the start index to put data into the 
    *  software transmit buffer.
    *  SCBUART_txBufferTail - start index to get data from the software
    *  transmit buffer.
    *
    *******************************************************************************/
    void SCBUART_SpiUartWriteTxData(uint32 txData)
    {
    #if (SCBUART_INTERNAL_TX_SW_BUFFER_CONST)
        uint32 locHead;
    #endif /* (SCBUART_INTERNAL_TX_SW_BUFFER_CONST) */

        #if (SCBUART_CHECK_TX_SW_BUFFER)
        {
            /* Put data directly into the TX FIFO */
            if ((SCBUART_txBufferHead == SCBUART_txBufferTail) &&
                (SCBUART_SPI_UART_FIFO_SIZE != SCBUART_GET_TX_FIFO_ENTRIES))
            {
                /* TX software buffer is empty: put data directly in TX FIFO */
                SCBUART_TX_FIFO_WR_REG = txData;
            }
            /* Put data into TX software buffer */
            else
            {
                /* Head index to put data */
                locHead = (SCBUART_txBufferHead + 1u);

                /* Adjust TX software buffer index */
                if (SCBUART_TX_BUFFER_SIZE == locHead)
                {
                    locHead = 0u;
                }

                /* Wait for space in TX software buffer */
                while (locHead == SCBUART_txBufferTail)
                {
                }

                /* TX software buffer has at least one room */

                /* Clear old status of INTR_TX_NOT_FULL. It sets at the end of transfer when TX FIFO is empty. */
                SCBUART_ClearTxInterruptSource(SCBUART_INTR_TX_NOT_FULL);

                SCBUART_PutWordInTxBuffer(locHead, txData);

                SCBUART_txBufferHead = locHead;

                /* Check if TX Not Full is disabled in interrupt */
                if (0u == (SCBUART_INTR_TX_MASK_REG & SCBUART_INTR_TX_NOT_FULL))
                {
                    /* Enable TX Not Full interrupt source to transmit from software buffer */
                    SCBUART_INTR_TX_MASK_REG |= (uint32) SCBUART_INTR_TX_NOT_FULL;
                }
            }
        }
        #else
        {
            /* Wait until TX FIFO has space to put data element */
            while (SCBUART_SPI_UART_FIFO_SIZE == SCBUART_GET_TX_FIFO_ENTRIES)
            {
            }

            SCBUART_TX_FIFO_WR_REG = txData;
        }
        #endif
    }


    /*******************************************************************************
    * Function Name: SCBUART_SpiUartPutArray
    ****************************************************************************//**
    *
    *  Places an array of data into the transmit buffer to be sent.
    *  This function is blocking and waits until there is a space available to put
    *  all the requested data in the transmit buffer. The array size can be greater
    *  than transmit buffer size.
    *
    * \param wrBuf: pointer to an array of data to be placed in transmit buffer. 
    *  The width of the data to be transmitted depends on TX data width selection 
    *  (the data bit counting starts from LSB for each array element).
    * \param count: number of data elements to be placed in the transmit buffer.
    *
    * \globalvars
    *  SCBUART_txBufferHead - the start index to put data into the 
    *  software transmit buffer.
    *  SCBUART_txBufferTail - start index to get data from the software
    *  transmit buffer.
    *
    *******************************************************************************/
    void SCBUART_SpiUartPutArray(const uint8 wrBuf[], uint32 count)
    {
        uint32 i;

        for (i=0u; i < count; i++)
        {
            SCBUART_SpiUartWriteTxData((uint32) wrBuf[i]);
        }
    }


    /*******************************************************************************
    * Function Name: SCBUART_SpiUartGetTxBufferSize
    ****************************************************************************//**
    *
    *  Returns the number of elements currently in the transmit buffer.
    *   - TX software buffer is disabled: returns the number of used entries in
    *     TX FIFO.
    *   - TX software buffer is enabled: returns the number of elements currently
    *     used in the transmit buffer. This number does not include used entries in
    *     the TX FIFO. The transmit buffer size is zero until the TX FIFO is
    *     not full.
    *
    * \return
    *  Number of data elements ready to transmit.
    *
    * \globalvars
    *  SCBUART_txBufferHead - the start index to put data into the 
    *  software transmit buffer.
    *  SCBUART_txBufferTail - start index to get data from the software
    *  transmit buffer.
    *
    *******************************************************************************/
    uint32 SCBUART_SpiUartGetTxBufferSize(void)
    {
        uint32 size;
    #if (SCBUART_INTERNAL_TX_SW_BUFFER_CONST)
        uint32 locTail;
    #endif /* (SCBUART_INTERNAL_TX_SW_BUFFER_CONST) */

        #if (SCBUART_CHECK_TX_SW_BUFFER)
        {
            /* Get current Tail index */
            locTail = SCBUART_txBufferTail;

            if (SCBUART_txBufferHead >= locTail)
            {
                size = (SCBUART_txBufferHead - locTail);
            }
            else
            {
                size = (SCBUART_txBufferHead + (SCBUART_TX_BUFFER_SIZE - locTail));
            }
        }
        #else
        {
            size = SCBUART_GET_TX_FIFO_ENTRIES;
        }
        #endif

        return (size);
    }


    /*******************************************************************************
    * Function Name: SCBUART_SpiUartClearTxBuffer
    ****************************************************************************//**
    *
    *  Clears the transmit buffer and TX FIFO.
    *
    * \globalvars
    *  SCBUART_txBufferHead - the start index to put data into the 
    *  software transmit buffer.
    *  SCBUART_txBufferTail - start index to get data from the software
    *  transmit buffer.
    *
    *******************************************************************************/
    void SCBUART_SpiUartClearTxBuffer(void)
    {
        #if (SCBUART_CHECK_TX_SW_BUFFER)
        {
            /* Lock from component interruption */
            SCBUART_DisableInt();

            /* Flush TX software buffer */
            SCBUART_txBufferHead = SCBUART_txBufferTail;

            SCBUART_INTR_TX_MASK_REG &= (uint32) ~SCBUART_INTR_TX_NOT_FULL;
            SCBUART_CLEAR_TX_FIFO;
            SCBUART_ClearTxInterruptSource(SCBUART_INTR_TX_ALL);

            /* Release lock */
            SCBUART_EnableInt();
        }
        #else
        {
            SCBUART_CLEAR_TX_FIFO;
        }
        #endif
    }

#endif /* (SCBUART_TX_DIRECTION) */


/*******************************************************************************
* Function Name: SCBUART_SpiUartDisableIntRx
****************************************************************************//**
*
*  Disables the RX interrupt sources.
*
*  \return
*   Returns the RX interrupt sources enabled before the function call.
*
*******************************************************************************/
uint32 SCBUART_SpiUartDisableIntRx(void)
{
    uint32 intSource;

    intSource = SCBUART_GetRxInterruptMode();

    SCBUART_SetRxInterruptMode(SCBUART_NO_INTR_SOURCES);

    return (intSource);
}


/*******************************************************************************
* Function Name: SCBUART_SpiUartDisableIntTx
****************************************************************************//**
*
*  Disables TX interrupt sources.
*
*  \return
*   Returns TX interrupt sources enabled before function call.
*
*******************************************************************************/
uint32 SCBUART_SpiUartDisableIntTx(void)
{
    uint32 intSourceMask;

    intSourceMask = SCBUART_GetTxInterruptMode();

    SCBUART_SetTxInterruptMode(SCBUART_NO_INTR_SOURCES);

    return (intSourceMask);
}


#if(SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: SCBUART_PutWordInRxBuffer
    ****************************************************************************//**
    *
    *  Stores a byte/word into the RX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    *  \param index:      index to store data byte/word in the RX buffer.
    *  \param rxDataByte: byte/word to store.
    *
    *******************************************************************************/
    void SCBUART_PutWordInRxBuffer(uint32 idx, uint32 rxDataByte)
    {
        /* Put data in buffer */
        if (SCBUART_ONE_BYTE_WIDTH == SCBUART_rxDataBits)
        {
            SCBUART_rxBuffer[idx] = ((uint8) rxDataByte);
        }
        else
        {
            SCBUART_rxBuffer[(uint32)(idx << 1u)]      = LO8(LO16(rxDataByte));
            SCBUART_rxBuffer[(uint32)(idx << 1u) + 1u] = HI8(LO16(rxDataByte));
        }
    }


    /*******************************************************************************
    * Function Name: SCBUART_GetWordFromRxBuffer
    ****************************************************************************//**
    *
    *  Reads byte/word from RX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    *  \return
    *   Returns byte/word read from RX buffer.
    *
    *******************************************************************************/
    uint32 SCBUART_GetWordFromRxBuffer(uint32 idx)
    {
        uint32 value;

        if (SCBUART_ONE_BYTE_WIDTH == SCBUART_rxDataBits)
        {
            value = SCBUART_rxBuffer[idx];
        }
        else
        {
            value  = (uint32) SCBUART_rxBuffer[(uint32)(idx << 1u)];
            value |= (uint32) ((uint32)SCBUART_rxBuffer[(uint32)(idx << 1u) + 1u] << 8u);
        }

        return (value);
    }


    /*******************************************************************************
    * Function Name: SCBUART_PutWordInTxBuffer
    ****************************************************************************//**
    *
    *  Stores byte/word into the TX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    *  \param idx:        index to store data byte/word in the TX buffer.
    *  \param txDataByte: byte/word to store.
    *
    *******************************************************************************/
    void SCBUART_PutWordInTxBuffer(uint32 idx, uint32 txDataByte)
    {
        /* Put data in buffer */
        if (SCBUART_ONE_BYTE_WIDTH == SCBUART_txDataBits)
        {
            SCBUART_txBuffer[idx] = ((uint8) txDataByte);
        }
        else
        {
            SCBUART_txBuffer[(uint32)(idx << 1u)]      = LO8(LO16(txDataByte));
            SCBUART_txBuffer[(uint32)(idx << 1u) + 1u] = HI8(LO16(txDataByte));
        }
    }


    /*******************************************************************************
    * Function Name: SCBUART_GetWordFromTxBuffer
    ****************************************************************************//**
    *
    *  Reads byte/word from the TX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    *  \param idx: index to get data byte/word from the TX buffer.
    *
    *  \return
    *   Returns byte/word read from the TX buffer.
    *
    *******************************************************************************/
    uint32 SCBUART_GetWordFromTxBuffer(uint32 idx)
    {
        uint32 value;

        if (SCBUART_ONE_BYTE_WIDTH == SCBUART_txDataBits)
        {
            value = (uint32) SCBUART_txBuffer[idx];
        }
        else
        {
            value  = (uint32) SCBUART_txBuffer[(uint32)(idx << 1u)];
            value |= (uint32) ((uint32) SCBUART_txBuffer[(uint32)(idx << 1u) + 1u] << 8u);
        }

        return (value);
    }

#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */


/* [] END OF FILE */
