/***************************************************************************//**
* \file SCBUART_SPI_UART_INT.c
* \version 3.20
*
* \brief
*  This file provides the source code to the Interrupt Service Routine for
*  the SCB Component in SPI and UART modes.
*
* Note:
*
********************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SCBUART_PVT.h"
#include "SCBUART_SPI_UART_PVT.h"
#include "cyapicallbacks.h"

#if (SCBUART_SCB_IRQ_INTERNAL)
/*******************************************************************************
* Function Name: SCBUART_SPI_UART_ISR
****************************************************************************//**
*
*  Handles the Interrupt Service Routine for the SCB SPI or UART modes.
*
*******************************************************************************/
CY_ISR(SCBUART_SPI_UART_ISR)
{
#if (SCBUART_INTERNAL_RX_SW_BUFFER_CONST)
    uint32 locHead;
#endif /* (SCBUART_INTERNAL_RX_SW_BUFFER_CONST) */

#if (SCBUART_INTERNAL_TX_SW_BUFFER_CONST)
    uint32 locTail;
#endif /* (SCBUART_INTERNAL_TX_SW_BUFFER_CONST) */

#ifdef SCBUART_SPI_UART_ISR_ENTRY_CALLBACK
    SCBUART_SPI_UART_ISR_EntryCallback();
#endif /* SCBUART_SPI_UART_ISR_ENTRY_CALLBACK */

    if (NULL != SCBUART_customIntrHandler)
    {
        SCBUART_customIntrHandler();
    }

    #if(SCBUART_CHECK_SPI_WAKE_ENABLE)
    {
        /* Clear SPI wakeup source */
        SCBUART_ClearSpiExtClkInterruptSource(SCBUART_INTR_SPI_EC_WAKE_UP);
    }
    #endif

    #if (SCBUART_CHECK_RX_SW_BUFFER)
    {
        if (SCBUART_CHECK_INTR_RX_MASKED(SCBUART_INTR_RX_NOT_EMPTY))
        {
            do
            {
                /* Move local head index */
                locHead = (SCBUART_rxBufferHead + 1u);

                /* Adjust local head index */
                if (SCBUART_INTERNAL_RX_BUFFER_SIZE == locHead)
                {
                    locHead = 0u;
                }

                if (locHead == SCBUART_rxBufferTail)
                {
                    #if (SCBUART_CHECK_UART_RTS_CONTROL_FLOW)
                    {
                        /* There is no space in the software buffer - disable the
                        * RX Not Empty interrupt source. The data elements are
                        * still being received into the RX FIFO until the RTS signal
                        * stops the transmitter. After the data element is read from the
                        * buffer, the RX Not Empty interrupt source is enabled to
                        * move the next data element in the software buffer.
                        */
                        SCBUART_INTR_RX_MASK_REG &= ~SCBUART_INTR_RX_NOT_EMPTY;
                        break;
                    }
                    #else
                    {
                        /* Overflow: through away received data element */
                        (void) SCBUART_RX_FIFO_RD_REG;
                        SCBUART_rxBufferOverflow = (uint8) SCBUART_INTR_RX_OVERFLOW;
                    }
                    #endif
                }
                else
                {
                    /* Store received data */
                    SCBUART_PutWordInRxBuffer(locHead, SCBUART_RX_FIFO_RD_REG);

                    /* Move head index */
                    SCBUART_rxBufferHead = locHead;
                }
            }
            while(0u != SCBUART_GET_RX_FIFO_ENTRIES);

            SCBUART_ClearRxInterruptSource(SCBUART_INTR_RX_NOT_EMPTY);
        }
    }
    #endif


    #if (SCBUART_CHECK_TX_SW_BUFFER)
    {
        if (SCBUART_CHECK_INTR_TX_MASKED(SCBUART_INTR_TX_NOT_FULL))
        {
            do
            {
                /* Check for room in TX software buffer */
                if (SCBUART_txBufferHead != SCBUART_txBufferTail)
                {
                    /* Move local tail index */
                    locTail = (SCBUART_txBufferTail + 1u);

                    /* Adjust local tail index */
                    if (SCBUART_TX_BUFFER_SIZE == locTail)
                    {
                        locTail = 0u;
                    }

                    /* Put data into TX FIFO */
                    SCBUART_TX_FIFO_WR_REG = SCBUART_GetWordFromTxBuffer(locTail);

                    /* Move tail index */
                    SCBUART_txBufferTail = locTail;
                }
                else
                {
                    /* TX software buffer is empty: complete transfer */
                    SCBUART_DISABLE_INTR_TX(SCBUART_INTR_TX_NOT_FULL);
                    break;
                }
            }
            while (SCBUART_SPI_UART_FIFO_SIZE != SCBUART_GET_TX_FIFO_ENTRIES);

            SCBUART_ClearTxInterruptSource(SCBUART_INTR_TX_NOT_FULL);
        }
    }
    #endif

#ifdef SCBUART_SPI_UART_ISR_EXIT_CALLBACK
    SCBUART_SPI_UART_ISR_ExitCallback();
#endif /* SCBUART_SPI_UART_ISR_EXIT_CALLBACK */

}

#endif /* (SCBUART_SCB_IRQ_INTERNAL) */


/* [] END OF FILE */
