/***************************************************************************//**
* \file SCBUART_UART.c
* \version 3.20
*
* \brief
*  This file provides the source code to the API for the SCB Component in
*  UART mode.
*
* Note:
*
*******************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SCBUART_PVT.h"
#include "SCBUART_SPI_UART_PVT.h"
#include "cyapicallbacks.h"

#if (SCBUART_UART_WAKE_ENABLE_CONST && SCBUART_UART_RX_WAKEUP_IRQ)
    /**
    * \addtogroup group_globals
    * \{
    */
    /** This global variable determines whether to enable Skip Start
    * functionality when SCBUART_Sleep() function is called:
    * 0  disable, other values  enable. Default value is 1.
    * It is only available when Enable wakeup from Deep Sleep Mode is enabled.
    */
    uint8 SCBUART_skipStart = 1u;
    /** \} globals */
#endif /* (SCBUART_UART_WAKE_ENABLE_CONST && SCBUART_UART_RX_WAKEUP_IRQ) */

#if(SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)

    /***************************************
    *  Configuration Structure Initialization
    ***************************************/

    const SCBUART_UART_INIT_STRUCT SCBUART_configUart =
    {
        SCBUART_UART_SUB_MODE,
        SCBUART_UART_DIRECTION,
        SCBUART_UART_DATA_BITS_NUM,
        SCBUART_UART_PARITY_TYPE,
        SCBUART_UART_STOP_BITS_NUM,
        SCBUART_UART_OVS_FACTOR,
        SCBUART_UART_IRDA_LOW_POWER,
        SCBUART_UART_MEDIAN_FILTER_ENABLE,
        SCBUART_UART_RETRY_ON_NACK,
        SCBUART_UART_IRDA_POLARITY,
        SCBUART_UART_DROP_ON_PARITY_ERR,
        SCBUART_UART_DROP_ON_FRAME_ERR,
        SCBUART_UART_WAKE_ENABLE,
        0u,
        NULL,
        0u,
        NULL,
        SCBUART_UART_MP_MODE_ENABLE,
        SCBUART_UART_MP_ACCEPT_ADDRESS,
        SCBUART_UART_MP_RX_ADDRESS,
        SCBUART_UART_MP_RX_ADDRESS_MASK,
        (uint32) SCBUART_SCB_IRQ_INTERNAL,
        SCBUART_UART_INTR_RX_MASK,
        SCBUART_UART_RX_TRIGGER_LEVEL,
        SCBUART_UART_INTR_TX_MASK,
        SCBUART_UART_TX_TRIGGER_LEVEL,
        (uint8) SCBUART_UART_BYTE_MODE_ENABLE,
        (uint8) SCBUART_UART_CTS_ENABLE,
        (uint8) SCBUART_UART_CTS_POLARITY,
        (uint8) SCBUART_UART_RTS_POLARITY,
        (uint8) SCBUART_UART_RTS_FIFO_LEVEL
    };


    /*******************************************************************************
    * Function Name: SCBUART_UartInit
    ****************************************************************************//**
    *
    *  Configures the SCBUART for UART operation.
    *
    *  This function is intended specifically to be used when the SCBUART
    *  configuration is set to Unconfigured SCBUART in the customizer.
    *  After initializing the SCBUART in UART mode using this function,
    *  the component can be enabled using the SCBUART_Start() or
    * SCBUART_Enable() function.
    *  This function uses a pointer to a structure that provides the configuration
    *  settings. This structure contains the same information that would otherwise
    *  be provided by the customizer settings.
    *
    *  \param config: pointer to a structure that contains the following list of
    *   fields. These fields match the selections available in the customizer.
    *   Refer to the customizer for further description of the settings.
    *
    *******************************************************************************/
    void SCBUART_UartInit(const SCBUART_UART_INIT_STRUCT *config)
    {
        uint32 pinsConfig;

        if (NULL == config)
        {
            CYASSERT(0u != 0u); /* Halt execution due to bad function parameter */
        }
        else
        {
            /* Get direction to configure UART pins: TX, RX or TX+RX */
            pinsConfig  = config->direction;

        #if !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1)
            /* Add RTS and CTS pins to configure */
            pinsConfig |= (0u != config->rtsRxFifoLevel) ? (SCBUART_UART_RTS_PIN_ENABLE) : (0u);
            pinsConfig |= (0u != config->enableCts)      ? (SCBUART_UART_CTS_PIN_ENABLE) : (0u);
        #endif /* !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1) */

            /* Configure pins */
            SCBUART_SetPins(SCBUART_SCB_MODE_UART, config->mode, pinsConfig);

            /* Store internal configuration */
            SCBUART_scbMode       = (uint8) SCBUART_SCB_MODE_UART;
            SCBUART_scbEnableWake = (uint8) config->enableWake;
            SCBUART_scbEnableIntr = (uint8) config->enableInterrupt;

            /* Set RX direction internal variables */
            SCBUART_rxBuffer      =         config->rxBuffer;
            SCBUART_rxDataBits    = (uint8) config->dataBits;
            SCBUART_rxBufferSize  = (uint8) config->rxBufferSize;

            /* Set TX direction internal variables */
            SCBUART_txBuffer      =         config->txBuffer;
            SCBUART_txDataBits    = (uint8) config->dataBits;
            SCBUART_txBufferSize  = (uint8) config->txBufferSize;

            /* Configure UART interface */
            if(SCBUART_UART_MODE_IRDA == config->mode)
            {
                /* OVS settings: IrDA */
                SCBUART_CTRL_REG  = ((0u != config->enableIrdaLowPower) ?
                                                (SCBUART_UART_GET_CTRL_OVS_IRDA_LP(config->oversample)) :
                                                (SCBUART_CTRL_OVS_IRDA_OVS16));
            }
            else
            {
                /* OVS settings: UART and SmartCard */
                SCBUART_CTRL_REG  = SCBUART_GET_CTRL_OVS(config->oversample);
            }

            SCBUART_CTRL_REG     |= SCBUART_GET_CTRL_BYTE_MODE  (config->enableByteMode)      |
                                             SCBUART_GET_CTRL_ADDR_ACCEPT(config->multiprocAcceptAddr) |
                                             SCBUART_CTRL_UART;

            /* Configure sub-mode: UART, SmartCard or IrDA */
            SCBUART_UART_CTRL_REG = SCBUART_GET_UART_CTRL_MODE(config->mode);

            /* Configure RX direction */
            SCBUART_UART_RX_CTRL_REG = SCBUART_GET_UART_RX_CTRL_MODE(config->stopBits)              |
                                        SCBUART_GET_UART_RX_CTRL_POLARITY(config->enableInvertedRx)          |
                                        SCBUART_GET_UART_RX_CTRL_MP_MODE(config->enableMultiproc)            |
                                        SCBUART_GET_UART_RX_CTRL_DROP_ON_PARITY_ERR(config->dropOnParityErr) |
                                        SCBUART_GET_UART_RX_CTRL_DROP_ON_FRAME_ERR(config->dropOnFrameErr);

            if(SCBUART_UART_PARITY_NONE != config->parity)
            {
               SCBUART_UART_RX_CTRL_REG |= SCBUART_GET_UART_RX_CTRL_PARITY(config->parity) |
                                                    SCBUART_UART_RX_CTRL_PARITY_ENABLED;
            }

            SCBUART_RX_CTRL_REG      = SCBUART_GET_RX_CTRL_DATA_WIDTH(config->dataBits)       |
                                                SCBUART_GET_RX_CTRL_MEDIAN(config->enableMedianFilter) |
                                                SCBUART_GET_UART_RX_CTRL_ENABLED(config->direction);

            SCBUART_RX_FIFO_CTRL_REG = SCBUART_GET_RX_FIFO_CTRL_TRIGGER_LEVEL(config->rxTriggerLevel);

            /* Configure MP address */
            SCBUART_RX_MATCH_REG     = SCBUART_GET_RX_MATCH_ADDR(config->multiprocAddr) |
                                                SCBUART_GET_RX_MATCH_MASK(config->multiprocAddrMask);

            /* Configure RX direction */
            SCBUART_UART_TX_CTRL_REG = SCBUART_GET_UART_TX_CTRL_MODE(config->stopBits) |
                                                SCBUART_GET_UART_TX_CTRL_RETRY_NACK(config->enableRetryNack);

            if(SCBUART_UART_PARITY_NONE != config->parity)
            {
               SCBUART_UART_TX_CTRL_REG |= SCBUART_GET_UART_TX_CTRL_PARITY(config->parity) |
                                                    SCBUART_UART_TX_CTRL_PARITY_ENABLED;
            }

            SCBUART_TX_CTRL_REG      = SCBUART_GET_TX_CTRL_DATA_WIDTH(config->dataBits)    |
                                                SCBUART_GET_UART_TX_CTRL_ENABLED(config->direction);

            SCBUART_TX_FIFO_CTRL_REG = SCBUART_GET_TX_FIFO_CTRL_TRIGGER_LEVEL(config->txTriggerLevel);

        #if !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1)
            SCBUART_UART_FLOW_CTRL_REG = SCBUART_GET_UART_FLOW_CTRL_CTS_ENABLE(config->enableCts) | \
                                            SCBUART_GET_UART_FLOW_CTRL_CTS_POLARITY (config->ctsPolarity)  | \
                                            SCBUART_GET_UART_FLOW_CTRL_RTS_POLARITY (config->rtsPolarity)  | \
                                            SCBUART_GET_UART_FLOW_CTRL_TRIGGER_LEVEL(config->rtsRxFifoLevel);
        #endif /* !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1) */

            /* Configure interrupt with UART handler but do not enable it */
            CyIntDisable    (SCBUART_ISR_NUMBER);
            CyIntSetPriority(SCBUART_ISR_NUMBER, SCBUART_ISR_PRIORITY);
            (void) CyIntSetVector(SCBUART_ISR_NUMBER, &SCBUART_SPI_UART_ISR);

            /* Configure WAKE interrupt */
        #if(SCBUART_UART_RX_WAKEUP_IRQ)
            CyIntDisable    (SCBUART_RX_WAKE_ISR_NUMBER);
            CyIntSetPriority(SCBUART_RX_WAKE_ISR_NUMBER, SCBUART_RX_WAKE_ISR_PRIORITY);
            (void) CyIntSetVector(SCBUART_RX_WAKE_ISR_NUMBER, &SCBUART_UART_WAKEUP_ISR);
        #endif /* (SCBUART_UART_RX_WAKEUP_IRQ) */

            /* Configure interrupt sources */
            SCBUART_INTR_I2C_EC_MASK_REG = SCBUART_NO_INTR_SOURCES;
            SCBUART_INTR_SPI_EC_MASK_REG = SCBUART_NO_INTR_SOURCES;
            SCBUART_INTR_SLAVE_MASK_REG  = SCBUART_NO_INTR_SOURCES;
            SCBUART_INTR_MASTER_MASK_REG = SCBUART_NO_INTR_SOURCES;
            SCBUART_INTR_RX_MASK_REG     = config->rxInterruptMask;
            SCBUART_INTR_TX_MASK_REG     = config->txInterruptMask;
        
            /* Configure TX interrupt sources to restore. */
            SCBUART_IntrTxMask = LO16(SCBUART_INTR_TX_MASK_REG);

            /* Clear RX buffer indexes */
            SCBUART_rxBufferHead     = 0u;
            SCBUART_rxBufferTail     = 0u;
            SCBUART_rxBufferOverflow = 0u;

            /* Clear TX buffer indexes */
            SCBUART_txBufferHead = 0u;
            SCBUART_txBufferTail = 0u;
        }
    }

#else

    /*******************************************************************************
    * Function Name: SCBUART_UartInit
    ****************************************************************************//**
    *
    *  Configures the SCB for the UART operation.
    *
    *******************************************************************************/
    void SCBUART_UartInit(void)
    {
        /* Configure UART interface */
        SCBUART_CTRL_REG = SCBUART_UART_DEFAULT_CTRL;

        /* Configure sub-mode: UART, SmartCard or IrDA */
        SCBUART_UART_CTRL_REG = SCBUART_UART_DEFAULT_UART_CTRL;

        /* Configure RX direction */
        SCBUART_UART_RX_CTRL_REG = SCBUART_UART_DEFAULT_UART_RX_CTRL;
        SCBUART_RX_CTRL_REG      = SCBUART_UART_DEFAULT_RX_CTRL;
        SCBUART_RX_FIFO_CTRL_REG = SCBUART_UART_DEFAULT_RX_FIFO_CTRL;
        SCBUART_RX_MATCH_REG     = SCBUART_UART_DEFAULT_RX_MATCH_REG;

        /* Configure TX direction */
        SCBUART_UART_TX_CTRL_REG = SCBUART_UART_DEFAULT_UART_TX_CTRL;
        SCBUART_TX_CTRL_REG      = SCBUART_UART_DEFAULT_TX_CTRL;
        SCBUART_TX_FIFO_CTRL_REG = SCBUART_UART_DEFAULT_TX_FIFO_CTRL;

    #if !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1)
        SCBUART_UART_FLOW_CTRL_REG = SCBUART_UART_DEFAULT_FLOW_CTRL;
    #endif /* !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1) */

        /* Configure interrupt with UART handler but do not enable it */
    #if(SCBUART_SCB_IRQ_INTERNAL)
        CyIntDisable    (SCBUART_ISR_NUMBER);
        CyIntSetPriority(SCBUART_ISR_NUMBER, SCBUART_ISR_PRIORITY);
        (void) CyIntSetVector(SCBUART_ISR_NUMBER, &SCBUART_SPI_UART_ISR);
    #endif /* (SCBUART_SCB_IRQ_INTERNAL) */

        /* Configure WAKE interrupt */
    #if(SCBUART_UART_RX_WAKEUP_IRQ)
        CyIntDisable    (SCBUART_RX_WAKE_ISR_NUMBER);
        CyIntSetPriority(SCBUART_RX_WAKE_ISR_NUMBER, SCBUART_RX_WAKE_ISR_PRIORITY);
        (void) CyIntSetVector(SCBUART_RX_WAKE_ISR_NUMBER, &SCBUART_UART_WAKEUP_ISR);
    #endif /* (SCBUART_UART_RX_WAKEUP_IRQ) */

        /* Configure interrupt sources */
        SCBUART_INTR_I2C_EC_MASK_REG = SCBUART_UART_DEFAULT_INTR_I2C_EC_MASK;
        SCBUART_INTR_SPI_EC_MASK_REG = SCBUART_UART_DEFAULT_INTR_SPI_EC_MASK;
        SCBUART_INTR_SLAVE_MASK_REG  = SCBUART_UART_DEFAULT_INTR_SLAVE_MASK;
        SCBUART_INTR_MASTER_MASK_REG = SCBUART_UART_DEFAULT_INTR_MASTER_MASK;
        SCBUART_INTR_RX_MASK_REG     = SCBUART_UART_DEFAULT_INTR_RX_MASK;
        SCBUART_INTR_TX_MASK_REG     = SCBUART_UART_DEFAULT_INTR_TX_MASK;
    
        /* Configure TX interrupt sources to restore. */
        SCBUART_IntrTxMask = LO16(SCBUART_INTR_TX_MASK_REG);

    #if(SCBUART_INTERNAL_RX_SW_BUFFER_CONST)
        SCBUART_rxBufferHead     = 0u;
        SCBUART_rxBufferTail     = 0u;
        SCBUART_rxBufferOverflow = 0u;
    #endif /* (SCBUART_INTERNAL_RX_SW_BUFFER_CONST) */

    #if(SCBUART_INTERNAL_TX_SW_BUFFER_CONST)
        SCBUART_txBufferHead = 0u;
        SCBUART_txBufferTail = 0u;
    #endif /* (SCBUART_INTERNAL_TX_SW_BUFFER_CONST) */
    }
#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */


/*******************************************************************************
* Function Name: SCBUART_UartPostEnable
****************************************************************************//**
*
*  Restores HSIOM settings for the UART output pins (TX and/or RTS) to be
*  controlled by the SCB UART.
*
*******************************************************************************/
void SCBUART_UartPostEnable(void)
{
#if (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)
    #if (SCBUART_TX_SDA_MISO_PIN)
        if (SCBUART_CHECK_TX_SDA_MISO_PIN_USED)
        {
            /* Set SCB UART to drive the output pin */
            SCBUART_SET_HSIOM_SEL(SCBUART_TX_SDA_MISO_HSIOM_REG, SCBUART_TX_SDA_MISO_HSIOM_MASK,
                                           SCBUART_TX_SDA_MISO_HSIOM_POS, SCBUART_TX_SDA_MISO_HSIOM_SEL_UART);
        }
    #endif /* (SCBUART_TX_SDA_MISO_PIN_PIN) */

    #if !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1)
        #if (SCBUART_RTS_SS0_PIN)
            if (SCBUART_CHECK_RTS_SS0_PIN_USED)
            {
                /* Set SCB UART to drive the output pin */
                SCBUART_SET_HSIOM_SEL(SCBUART_RTS_SS0_HSIOM_REG, SCBUART_RTS_SS0_HSIOM_MASK,
                                               SCBUART_RTS_SS0_HSIOM_POS, SCBUART_RTS_SS0_HSIOM_SEL_UART);
            }
        #endif /* (SCBUART_RTS_SS0_PIN) */
    #endif /* !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1) */

#else
    #if (SCBUART_UART_TX_PIN)
         /* Set SCB UART to drive the output pin */
        SCBUART_SET_HSIOM_SEL(SCBUART_TX_HSIOM_REG, SCBUART_TX_HSIOM_MASK,
                                       SCBUART_TX_HSIOM_POS, SCBUART_TX_HSIOM_SEL_UART);
    #endif /* (SCBUART_UART_TX_PIN) */

    #if (SCBUART_UART_RTS_PIN)
        /* Set SCB UART to drive the output pin */
        SCBUART_SET_HSIOM_SEL(SCBUART_RTS_HSIOM_REG, SCBUART_RTS_HSIOM_MASK,
                                       SCBUART_RTS_HSIOM_POS, SCBUART_RTS_HSIOM_SEL_UART);
    #endif /* (SCBUART_UART_RTS_PIN) */
#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */

    /* Restore TX interrupt sources. */
    SCBUART_SetTxInterruptMode(SCBUART_IntrTxMask);
}


/*******************************************************************************
* Function Name: SCBUART_UartStop
****************************************************************************//**
*
*  Changes the HSIOM settings for the UART output pins (TX and/or RTS) to keep
*  them inactive after the block is disabled. The output pins are controlled by
*  the GPIO data register. Also, the function disables the skip start feature
*  to not cause it to trigger after the component is enabled.
*
*******************************************************************************/
void SCBUART_UartStop(void)
{
#if(SCBUART_SCB_MODE_UNCONFIG_CONST_CFG)
    #if (SCBUART_TX_SDA_MISO_PIN)
        if (SCBUART_CHECK_TX_SDA_MISO_PIN_USED)
        {
            /* Set GPIO to drive output pin */
            SCBUART_SET_HSIOM_SEL(SCBUART_TX_SDA_MISO_HSIOM_REG, SCBUART_TX_SDA_MISO_HSIOM_MASK,
                                           SCBUART_TX_SDA_MISO_HSIOM_POS, SCBUART_TX_SDA_MISO_HSIOM_SEL_GPIO);
        }
    #endif /* (SCBUART_TX_SDA_MISO_PIN_PIN) */

    #if !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1)
        #if (SCBUART_RTS_SS0_PIN)
            if (SCBUART_CHECK_RTS_SS0_PIN_USED)
            {
                /* Set output pin state after block is disabled */
                SCBUART_uart_rts_spi_ss0_Write(SCBUART_GET_UART_RTS_INACTIVE);

                /* Set GPIO to drive output pin */
                SCBUART_SET_HSIOM_SEL(SCBUART_RTS_SS0_HSIOM_REG, SCBUART_RTS_SS0_HSIOM_MASK,
                                               SCBUART_RTS_SS0_HSIOM_POS, SCBUART_RTS_SS0_HSIOM_SEL_GPIO);
            }
        #endif /* (SCBUART_RTS_SS0_PIN) */
    #endif /* !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1) */

#else
    #if (SCBUART_UART_TX_PIN)
        /* Set GPIO to drive output pin */
        SCBUART_SET_HSIOM_SEL(SCBUART_TX_HSIOM_REG, SCBUART_TX_HSIOM_MASK,
                                       SCBUART_TX_HSIOM_POS, SCBUART_TX_HSIOM_SEL_GPIO);
    #endif /* (SCBUART_UART_TX_PIN) */

    #if (SCBUART_UART_RTS_PIN)
        /* Set output pin state after block is disabled */
        SCBUART_rts_Write(SCBUART_GET_UART_RTS_INACTIVE);

        /* Set GPIO to drive output pin */
        SCBUART_SET_HSIOM_SEL(SCBUART_RTS_HSIOM_REG, SCBUART_RTS_HSIOM_MASK,
                                       SCBUART_RTS_HSIOM_POS, SCBUART_RTS_HSIOM_SEL_GPIO);
    #endif /* (SCBUART_UART_RTS_PIN) */

#endif /* (SCBUART_SCB_MODE_UNCONFIG_CONST_CFG) */

#if (SCBUART_UART_WAKE_ENABLE_CONST)
    /* Disable skip start feature used for wakeup */
    SCBUART_UART_RX_CTRL_REG &= (uint32) ~SCBUART_UART_RX_CTRL_SKIP_START;
#endif /* (SCBUART_UART_WAKE_ENABLE_CONST) */

    /* Store TX interrupt sources (exclude level triggered). */
    SCBUART_IntrTxMask = LO16(SCBUART_GetTxInterruptMode() & SCBUART_INTR_UART_TX_RESTORE);
}


/*******************************************************************************
* Function Name: SCBUART_UartSetRxAddress
****************************************************************************//**
*
*  Sets the hardware detectable receiver address for the UART in the
*  Multiprocessor mode.
*
*  \param address: Address for hardware address detection.
*
*******************************************************************************/
void SCBUART_UartSetRxAddress(uint32 address)
{
     uint32 matchReg;

    matchReg = SCBUART_RX_MATCH_REG;

    matchReg &= ((uint32) ~SCBUART_RX_MATCH_ADDR_MASK); /* Clear address bits */
    matchReg |= ((uint32)  (address & SCBUART_RX_MATCH_ADDR_MASK)); /* Set address  */

    SCBUART_RX_MATCH_REG = matchReg;
}


/*******************************************************************************
* Function Name: SCBUART_UartSetRxAddressMask
****************************************************************************//**
*
*  Sets the hardware address mask for the UART in the Multiprocessor mode.
*
*  \param addressMask: Address mask.
*   - Bit value 0  excludes bit from address comparison.
*   - Bit value 1  the bit needs to match with the corresponding bit
*     of the address.
*
*******************************************************************************/
void SCBUART_UartSetRxAddressMask(uint32 addressMask)
{
    uint32 matchReg;

    matchReg = SCBUART_RX_MATCH_REG;

    matchReg &= ((uint32) ~SCBUART_RX_MATCH_MASK_MASK); /* Clear address mask bits */
    matchReg |= ((uint32) (addressMask << SCBUART_RX_MATCH_MASK_POS));

    SCBUART_RX_MATCH_REG = matchReg;
}


#if(SCBUART_UART_RX_DIRECTION)
    /*******************************************************************************
    * Function Name: SCBUART_UartGetChar
    ****************************************************************************//**
    *
    *  Retrieves next data element from receive buffer.
    *  This function is designed for ASCII characters and returns a char where
    *  1 to 255 are valid characters and 0 indicates an error occurred or no data
    *  is present.
    *  - RX software buffer is disabled: Returns data element retrieved from RX
    *    FIFO.
    *  - RX software buffer is enabled: Returns data element from the software
    *    receive buffer.
    *
    *  \return
    *   Next data element from the receive buffer. ASCII character values from
    *   1 to 255 are valid. A returned zero signifies an error condition or no
    *   data available.
    *
    *  \sideeffect
    *   The errors bits may not correspond with reading characters due to
    *   RX FIFO and software buffer usage.
    *   RX software buffer is enabled: The internal software buffer overflow
    *   is not treated as an error condition.
    *   Check SCBUART_rxBufferOverflow to capture that error condition.
    *
    *******************************************************************************/
    uint32 SCBUART_UartGetChar(void)
    {
        uint32 rxData = 0u;

        /* Reads data only if there is data to read */
        if (0u != SCBUART_SpiUartGetRxBufferSize())
        {
            rxData = SCBUART_SpiUartReadRxData();
        }

        if (SCBUART_CHECK_INTR_RX(SCBUART_INTR_RX_ERR))
        {
            rxData = 0u; /* Error occurred: returns zero */
            SCBUART_ClearRxInterruptSource(SCBUART_INTR_RX_ERR);
        }

        return (rxData);
    }


    /*******************************************************************************
    * Function Name: SCBUART_UartGetByte
    ****************************************************************************//**
    *
    *  Retrieves the next data element from the receive buffer, returns the
    *  received byte and error condition.
    *   - The RX software buffer is disabled: returns the data element retrieved
    *     from the RX FIFO. Undefined data will be returned if the RX FIFO is
    *     empty.
    *   - The RX software buffer is enabled: returns data element from the
    *     software receive buffer.
    *
    *  \return
    *   Bits 7-0 contain the next data element from the receive buffer and
    *   other bits contain the error condition.
    *   - SCBUART_UART_RX_OVERFLOW - Attempt to write to a full
    *     receiver FIFO.
    *   - SCBUART_UART_RX_UNDERFLOW	Attempt to read from an empty
    *     receiver FIFO.
    *   - SCBUART_UART_RX_FRAME_ERROR - UART framing error detected.
    *   - SCBUART_UART_RX_PARITY_ERROR - UART parity error detected.
    *
    *  \sideeffect
    *   The errors bits may not correspond with reading characters due to
    *   RX FIFO and software buffer usage.
    *   RX software buffer is enabled: The internal software buffer overflow
    *   is not treated as an error condition.
    *   Check SCBUART_rxBufferOverflow to capture that error condition.
    *
    *******************************************************************************/
    uint32 SCBUART_UartGetByte(void)
    {
        uint32 rxData;
        uint32 tmpStatus;

        #if (SCBUART_CHECK_RX_SW_BUFFER)
        {
            SCBUART_DisableInt();
        }
        #endif

        if (0u != SCBUART_SpiUartGetRxBufferSize())
        {
            /* Enables interrupt to receive more bytes: at least one byte is in
            * buffer.
            */
            #if (SCBUART_CHECK_RX_SW_BUFFER)
            {
                SCBUART_EnableInt();
            }
            #endif

            /* Get received byte */
            rxData = SCBUART_SpiUartReadRxData();
        }
        else
        {
            /* Reads a byte directly from RX FIFO: underflow is raised in the
            * case of empty. Otherwise the first received byte will be read.
            */
            rxData = SCBUART_RX_FIFO_RD_REG;


            /* Enables interrupt to receive more bytes. */
            #if (SCBUART_CHECK_RX_SW_BUFFER)
            {

                /* The byte has been read from RX FIFO. Clear RX interrupt to
                * not involve interrupt handler when RX FIFO is empty.
                */
                SCBUART_ClearRxInterruptSource(SCBUART_INTR_RX_NOT_EMPTY);

                SCBUART_EnableInt();
            }
            #endif
        }

        /* Get and clear RX error mask */
        tmpStatus = (SCBUART_GetRxInterruptSource() & SCBUART_INTR_RX_ERR);
        SCBUART_ClearRxInterruptSource(SCBUART_INTR_RX_ERR);

        /* Puts together data and error status:
        * MP mode and accept address: 9th bit is set to notify mark.
        */
        rxData |= ((uint32) (tmpStatus << 8u));

        return (rxData);
    }


    #if !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1)
        /*******************************************************************************
        * Function Name: SCBUART_UartSetRtsPolarity
        ****************************************************************************//**
        *
        *  Sets active polarity of RTS output signal.
        *  Only available for PSoC 4100 BLE / PSoC 4200 BLE / PSoC 4100M / PSoC 4200M /
        *  PSoC 4200L / PSoC 4000S / PSoC 4100S / PSoC Analog Coprocessor devices.
        *
        *  \param polarity: Active polarity of RTS output signal.
        *   - SCBUART_UART_RTS_ACTIVE_LOW  - RTS signal is active low.
        *   - SCBUART_UART_RTS_ACTIVE_HIGH - RTS signal is active high.
        *
        *******************************************************************************/
        void SCBUART_UartSetRtsPolarity(uint32 polarity)
        {
            if(0u != polarity)
            {
                SCBUART_UART_FLOW_CTRL_REG |= (uint32)  SCBUART_UART_FLOW_CTRL_RTS_POLARITY;
            }
            else
            {
                SCBUART_UART_FLOW_CTRL_REG &= (uint32) ~SCBUART_UART_FLOW_CTRL_RTS_POLARITY;
            }
        }


        /*******************************************************************************
        * Function Name: SCBUART_UartSetRtsFifoLevel
        ****************************************************************************//**
        *
        *  Sets level in the RX FIFO for RTS signal activation.
        *  While the RX FIFO has fewer entries than the RX FIFO level the RTS signal
        *  remains active, otherwise the RTS signal becomes inactive.
        *  Only available for PSoC 4100 BLE / PSoC 4200 BLE / PSoC 4100M / PSoC 4200M /
        *  PSoC 4200L / PSoC 4000S / PSoC 4100S / PSoC Analog Coprocessor devices.
        *
        *  \param level: Level in the RX FIFO for RTS signal activation.
        *   The range of valid level values is between 0 and RX FIFO depth - 1.
        *   Setting level value to 0 disables RTS signal activation.
        *
        *******************************************************************************/
        void SCBUART_UartSetRtsFifoLevel(uint32 level)
        {
            uint32 uartFlowCtrl;

            uartFlowCtrl = SCBUART_UART_FLOW_CTRL_REG;

            uartFlowCtrl &= ((uint32) ~SCBUART_UART_FLOW_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
            uartFlowCtrl |= ((uint32) (SCBUART_UART_FLOW_CTRL_TRIGGER_LEVEL_MASK & level));

            SCBUART_UART_FLOW_CTRL_REG = uartFlowCtrl;
        }
    #endif /* !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1) */

#endif /* (SCBUART_UART_RX_DIRECTION) */


#if(SCBUART_UART_TX_DIRECTION)
    /*******************************************************************************
    * Function Name: SCBUART_UartPutString
    ****************************************************************************//**
    *
    *  Places a NULL terminated string in the transmit buffer to be sent at the
    *  next available bus time.
    *  This function is blocking and waits until there is a space available to put
    *  requested data in transmit buffer.
    *
    *  \param string: pointer to the null terminated string array to be placed in the
    *   transmit buffer.
    *
    *******************************************************************************/
    void SCBUART_UartPutString(const char8 string[])
    {
        uint32 bufIndex;

        bufIndex = 0u;

        /* Blocks the control flow until all data has been sent */
        while(string[bufIndex] != ((char8) 0))
        {
            SCBUART_UartPutChar((uint32) string[bufIndex]);
            bufIndex++;
        }
    }


    /*******************************************************************************
    * Function Name: SCBUART_UartPutCRLF
    ****************************************************************************//**
    *
    *  Places byte of data followed by a carriage return (0x0D) and line feed
    *  (0x0A) in the transmit buffer.
    *  This function is blocking and waits until there is a space available to put
    *  all requested data in transmit buffer.
    *
    *  \param txDataByte: the data to be transmitted.
    *
    *******************************************************************************/
    void SCBUART_UartPutCRLF(uint32 txDataByte)
    {
        SCBUART_UartPutChar(txDataByte);  /* Blocks control flow until all data has been sent */
        SCBUART_UartPutChar(0x0Du);       /* Blocks control flow until all data has been sent */
        SCBUART_UartPutChar(0x0Au);       /* Blocks control flow until all data has been sent */
    }


    #if !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1)
        /*******************************************************************************
        * Function Name: SCBUARTSCB_UartEnableCts
        ****************************************************************************//**
        *
        *  Enables usage of CTS input signal by the UART transmitter.
        *  Only available for PSoC 4100 BLE / PSoC 4200 BLE / PSoC 4100M / PSoC 4200M /
        *  PSoC 4200L / PSoC 4000S / PSoC 4100S / PSoC Analog Coprocessor devices.
        *
        *******************************************************************************/
        void SCBUART_UartEnableCts(void)
        {
            SCBUART_UART_FLOW_CTRL_REG |= (uint32)  SCBUART_UART_FLOW_CTRL_CTS_ENABLE;
        }


        /*******************************************************************************
        * Function Name: SCBUART_UartDisableCts
        ****************************************************************************//**
        *
        *  Disables usage of CTS input signal by the UART transmitter.
        *  Only available for PSoC 4100 BLE / PSoC 4200 BLE / PSoC 4100M / PSoC 4200M /
        *  PSoC 4200L / PSoC 4000S / PSoC 4100S / PSoC Analog Coprocessor devices.
        *
        *******************************************************************************/
        void SCBUART_UartDisableCts(void)
        {
            SCBUART_UART_FLOW_CTRL_REG &= (uint32) ~SCBUART_UART_FLOW_CTRL_CTS_ENABLE;
        }


        /*******************************************************************************
        * Function Name: SCBUART_UartSetCtsPolarity
        ****************************************************************************//**
        *
        *  Sets active polarity of CTS input signal.
        *  Only available for PSoC 4100 BLE / PSoC 4200 BLE / PSoC 4100M / PSoC 4200M /
        *  PSoC 4200L / PSoC 4000S / PSoC 4100S / PSoC Analog Coprocessor devices.
        *
        *  \param polarity: Active polarity of CTS output signal.
        *   - SCBUART_UART_CTS_ACTIVE_LOW  - CTS signal is active low.
        *   - SCBUART_UART_CTS_ACTIVE_HIGH - CTS signal is active high.
        *
        *******************************************************************************/
        void SCBUART_UartSetCtsPolarity(uint32 polarity)
        {
            if (0u != polarity)
            {
                SCBUART_UART_FLOW_CTRL_REG |= (uint32)  SCBUART_UART_FLOW_CTRL_CTS_POLARITY;
            }
            else
            {
                SCBUART_UART_FLOW_CTRL_REG &= (uint32) ~SCBUART_UART_FLOW_CTRL_CTS_POLARITY;
            }
        }
    #endif /* !(SCBUART_CY_SCBIP_V0 || SCBUART_CY_SCBIP_V1) */

#endif /* (SCBUART_UART_TX_DIRECTION) */


#if (SCBUART_UART_WAKE_ENABLE_CONST)
    /*******************************************************************************
    * Function Name: SCBUART_UartSaveConfig
    ****************************************************************************//**
    *
    *  Clears and enables an interrupt on a falling edge of the Rx input. The GPIO
    *  interrupt does not track in the active mode, therefore requires to be 
    *  cleared by this API.
    *
    *******************************************************************************/
    void SCBUART_UartSaveConfig(void)
    {
    #if (SCBUART_UART_RX_WAKEUP_IRQ)
        /* Set SKIP_START if requested (set by default). */
        if (0u != SCBUART_skipStart)
        {
            SCBUART_UART_RX_CTRL_REG |= (uint32)  SCBUART_UART_RX_CTRL_SKIP_START;
        }
        else
        {
            SCBUART_UART_RX_CTRL_REG &= (uint32) ~SCBUART_UART_RX_CTRL_SKIP_START;
        }
        
        /* Clear RX GPIO interrupt status and pending interrupt in NVIC because
        * falling edge on RX line occurs while UART communication in active mode.
        * Enable interrupt: next interrupt trigger should wakeup device.
        */
        SCBUART_CLEAR_UART_RX_WAKE_INTR;
        SCBUART_RxWakeClearPendingInt();
        SCBUART_RxWakeEnableInt();
    #endif /* (SCBUART_UART_RX_WAKEUP_IRQ) */
    }


    /*******************************************************************************
    * Function Name: SCBUART_UartRestoreConfig
    ****************************************************************************//**
    *
    *  Disables the RX GPIO interrupt. Until this function is called the interrupt
    *  remains active and triggers on every falling edge of the UART RX line.
    *
    *******************************************************************************/
    void SCBUART_UartRestoreConfig(void)
    {
    #if (SCBUART_UART_RX_WAKEUP_IRQ)
        /* Disable interrupt: no more triggers in active mode */
        SCBUART_RxWakeDisableInt();
    #endif /* (SCBUART_UART_RX_WAKEUP_IRQ) */
    }


    #if (SCBUART_UART_RX_WAKEUP_IRQ)
        /*******************************************************************************
        * Function Name: SCBUART_UART_WAKEUP_ISR
        ****************************************************************************//**
        *
        *  Handles the Interrupt Service Routine for the SCB UART mode GPIO wakeup
        *  event. This event is configured to trigger on a falling edge of the RX line.
        *
        *******************************************************************************/
        CY_ISR(SCBUART_UART_WAKEUP_ISR)
        {
        #ifdef SCBUART_UART_WAKEUP_ISR_ENTRY_CALLBACK
            SCBUART_UART_WAKEUP_ISR_EntryCallback();
        #endif /* SCBUART_UART_WAKEUP_ISR_ENTRY_CALLBACK */

            SCBUART_CLEAR_UART_RX_WAKE_INTR;

        #ifdef SCBUART_UART_WAKEUP_ISR_EXIT_CALLBACK
            SCBUART_UART_WAKEUP_ISR_ExitCallback();
        #endif /* SCBUART_UART_WAKEUP_ISR_EXIT_CALLBACK */
        }
    #endif /* (SCBUART_UART_RX_WAKEUP_IRQ) */
#endif /* (SCBUART_UART_RX_WAKEUP_IRQ) */


/* [] END OF FILE */
